#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
from time import sleep

from homevee.db_utils import get_database_con


def remind_users():
    db = get_database_con()

    with db:
        #query events and remind users of upcoming calendar entries

        return


def init_thread():
    #wait until midnight + 5 minutes
    t = datetime.datetime.today()

    remind_users()

    sleep(60)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import traceback
import urllib
import urllib2
import random

import homevee.VoiceAssistant
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.voice_patterns import PATTERN_JOKE


def get_pattern(db):
    return PATTERN_JOKE

def get_label():
    return "joke"

def run_command(username, text, context, db):
    return get_joke(username, text, context, db)

def get_joke(username, text, context, db):
    try:
        url = helper.SMART_API_PATH + "/?action=joke&text=" + urllib.quote(text.encode('utf8'))
        print url
        data = urllib2.urlopen(url).read()
    except Exception, e:
        traceback.print_exc()
        data = None

    if data is not None:
        return {'msg_speech': data, 'msg_text': data}
    else:
        result = get_error()
        return {'msg_speech': result, 'msg_text': result}

def get_error():
    return random.choice([
        'Mir fällt gerade kein Witz ein.'
    ])


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.Functions import sensor_data
from homevee.Functions.sensor_data import SENSOR_TYPE_MAP
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.context_keys import *
from homevee.VoiceAssistant.voice_patterns import PATTERN_SENSOR_DATA
from homevee.device_types import *

def get_pattern(db):
    return PATTERN_SENSOR_DATA

def get_label():
    return "sensor"

def run_command(username, text, context, db):
    return get_sensor_data(username, text, context, db)

def get_sensor_data(username, text, context, db):
    room = helper.find_room(text, db)

    sensor_type = find_sensor_type(text)

    if sensor_type is None:
        if context is not None and context:
            sensor_type = context['sensor_type']

    if room is None:
        if context is None:
            answer_data = [
                ['Welchen Raum ', ['meinst du', 'möchtest du abfragen'], '?']
            ]
            output = helper.generate_string(answer_data)

            helper.set_context(username, GET_SENSOR_DATA, {'sensor_type': sensor_type}, db)

            return {'msg_speech': output, 'msg_text': output}
        else:
            if 'room' in context:
                room = context['room']
                print(room)

    if sensor_type in SENSOR_TYPE_MAP:
        einheit = SENSOR_TYPE_MAP[sensor_type]['einheit_word']
    else:
        answer_data = [
            ['Diesen Wert kann ich noch nicht abfragen.'],
            ['Dieser Wert kann noch nicht abgefragt werden.']
        ]
        output = helper.generate_string(answer_data)
        return {'msg_speech': output, 'msg_text': output}


    values = []

    with db:
        cur = db.cursor()

        data = [
            {'type': ZWAVE_SENSOR, 'table': 'ZWAVE_SENSOREN', 'id_col': 'ID', 'room_col': 'RAUM', 'type_col': 'SENSOR_TYPE'},
            {'type': MQTT_SENSOR, 'table': 'MQTT_SENSORS', 'id_col': 'ID', 'room_col': 'ROOM', 'type_col': 'TYPE'}
        ]

        for item in data:
            cur.execute("SELECT * FROM "+item['table']+" WHERE "+item['type_col']+" = :type AND "+item['room_col']+" = :room",
                        {'type': sensor_type, 'room': room['LOCATION']})

            for sensor in cur.fetchall():
                values.append(sensor_data.get_sensor_data(username, room['LOCATION'], item['type'], sensor[item['id_col']], False, db))

    if len(values) > 0:
        #compute average value of all sensors in this room
        sum = 0

        try:
            for value in values:
                sum += float(value)
        except Exception:
            answer_data = [
                ['Im Raum ' + room['NAME'], ' gibt es noch keine ', ['Messungen', 'Werte', 'Sensorwerte'],'.']
            ]

            output = helper.generate_string(answer_data)

            return {'msg_speech': output, 'msg_text': output}

        if sensor_type not in ['powermeter']:
            avg_value = sum/len(values)
            value_word = str(avg_value) + ' ' + einheit
            answer_data = [
                ['Im Raum ' + room['NAME'], ' hat es ' + value_word, '.']
            ]
        else:
            value_word = str(int(sum)) + ' ' + einheit
            answer_data = [
                ['Im Raum ' + room['NAME'], ' liegt der  ', ['Verbrauch', 'Stromverbrauch'],' bei ', value_word, '.'],
                ['Der Raum ' + room['NAME'], ' verbraucht aktuell  ', value_word, '.']
            ]

        output = helper.generate_string(answer_data)

        # helper.set_context(username, GET_SENSOR_DATA, {'room': room}, db)
    else:
        answer_data = [
            ['Ich konnte ', ['hier', 'in diesem Raum'] ,' keine ', ['Daten', 'Sensoren', 'Sensorwerte'], ' finden.']
        ]
        output = helper.generate_string(answer_data)

    return {'msg_speech': output, 'msg_text': output}

def get_reed_data(username, text, context, db):
    room = helper.find_room(text, db)

    output = 'Reed'
    return {'msg_speech': output, 'msg_text': output}

def get_presence_data(username, text, context, db):
    room = helper.find_room(text, db)

    output = 'Anwesenheit'
    return {'msg_speech': output, 'msg_text': output}

def find_sensor_type(text):
    words = text.split()

    type_keywords = [
        {'type': 'temp', 'items': ['warm', 'kalt', 'temperatur']},
        {'type': 'hygro', 'items': ['feucht', 'luftfeuchtigkeit', 'luftfeuchte']},
        {'type': 'helligkeit', 'items': ['hell', 'helligkeit']},
        {'type': 'uv', 'items': ['uv-licht']},
        {'type': 'powermeter', 'items': ['verbrauch', 'stromverbrauch']}
    ]

    for type_keyword in type_keywords:
        for keyword in type_keyword['items']:
            if keyword in words:
                return type_keyword['type']

    return None


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.Functions.chefkoch_api import search_recipes
from homevee.VoiceAssistant.voice_patterns import PATTERN_RECIPES

def get_pattern(db):
    return PATTERN_RECIPES

def get_label():
    return "recipes"

def run_command(username, text, context, db):
    return voice_recipes(username, text, context, db)

def voice_recipes(username, text, context, db):
    words = text.split(" ")

    keyword = None
    found = False
    for word in words:
        if found:
            if keyword is None:
                keyword = word
            else:
                keyword = keyword + " " + word

        if word in ['über', 'für', 'zu', 'von']:
            found = True

    recipes = search_recipes(keyword, 5)

    recipe_data = []

    for item in recipes:
        recipe = item['recipe']

        recipe_data.append({'name':recipe['title']})

    text = None
    for item in recipe_data:
        if text is None:
            text = item['name']
        else:
            text = text + ", " + item['name']

    return {'msg_speech': 'Rezepte: '+text, 'msg_text': 'Rezepte: '+text}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import os
import socket
import ssl
import sys
import time
import traceback
from thread import start_new_thread
from websocket_server import WebsocketServer
import API
import AESCipher
import constants
import cronjobs
import webinterface
from DeviceAPI import mqtt_api
#from Functions import ar_control, people_classifier
from Functions.chat import send_chat_message
#from Helper import compression
from Helper.helper_functions import save_request_to_db, parse_http_headers, send_to_client, \
    generate_cert, update_ip_thread, update_cert_thread, update_cert, check_cert
from API import process_data
import VoiceAssistant
from cloud_communication import connect_to_cloud, cloud_connection_loop
from constants import END_OF_MESSAGE
from cronjobs import heating_scheme
from cronjobs.overview_notification import send_overview_notifications
from db_utils import get_database_con, set_server_data
from firebase_utils import send_notification_to_users
from smart_speaker_communication import smart_speaker_loop

HOST = ''  # Symbolic name meaning all available interfaces
PORT = 7777  # Arbitrary non-privileged port

#Zeitzone anpassen
os.environ['TZ'] = 'Europe/Berlin'

'''while True:
    data = raw_input("Data: ")
    key = raw_input("Key: ")

    aes = AESCipher.AESCipher(key)
    new_aes = AESCipher.AESCipher(key)
    print("Data: "+data)
    cipher = aes.encrypt(data)
    print("Cipher: "+cipher)
    plain = new_aes.decrypt(cipher)
    print("Plain: "+plain)'''

#aes = AESCipher.AESCipher('3s6v9y$B&E)H@McQfTjWmZq4t7w!z%C*')
#print("Plain: "+aes.decrypt('qYqXviw/9yowSeMeJqkRhC6QUtQdPjFhyUdlzb0Cyz4xddOQbLMpNUjs8hMbk8al'))

#cronjobs.save_energy_data.init_thread()

test_speech = True
if test_speech:
    while(True):
        input = raw_input("Gib einen Sprachbefehl ein: ")
        print(VoiceAssistant.voice_command_cloud("sascha", input, None, None, get_database_con()))

def websocket_client(client, server):
    server.set_fn_message_received(handle_websocket_msg)

def start_websocket_server():
    server = WebsocketServer(7889, host='')
    server.set_fn_new_client(websocket_client)
    server.run_forever()

def handle_websocket_msg(client, server, data):
    # infinite loop so that function do not terminate and thread do not end.
    db = get_database_con()

    sent_data = None

    while True:
        try:
            # Receiving from client
            method_start_time = time.time()

            data = data.decode("utf-8")

            if data == "":
                db.close()
                return

            print("Received: " + str(data))

            is_http = False
            is_resend = False

            error = False

            # Ist HTTP-Aufruf?
            try:
                data = parse_http_headers(data)
                is_http = True
            except AttributeError:
                try:
                    data = json.loads(data)
                    data = json.loads(data['msg'])
                    print("Request is not in HTTP-Format")
                except:
                    print("Data could not be parsed")
                    server.send_message(client, json.dumps({'status': 'error'}))
                    error = True

            '''if 'resend' in data and data['resend'] is True:
                # Daten erneut senden
                print("Resend!")
                is_resend = True
                print(str(len(sent_data)) + " | " + sent_data)

                send_to_client(sent_data[data['resendstart']:], conn, is_http)'''

            if not error and not is_resend:
                if not is_resend:
                    reply = process_data(data, db)

                    reply['computing_time'] = time.time() - method_start_time

                if (reply is not None):
                    server.send_message(client, json.dumps(reply))

                    save_request_to_db(data, reply, db)
                else:
                    print("Error in reply")
                    server.send_message(client, json.dumps({'status': 'error'}))

                if not data:
                    break
        # except socket.error:
        #    break
        except Exception as e:
            traceback.print_exc()
            print("Fehler: " + str(e))
            server.send_message(client, json.dumps({'status': 'error'}))

        # Stop endless loop
        break

    # came out of loop
    db.commit()
    db.close()
    print("Verbindung beendet")

# Function for handling connections. This will be used to create threads
def clientthread(conn):
    # infinite loop so that function do not terminate and thread do not end.
    db = get_database_con()

    is_http = False

    sent_data = None

    while True:
        try:
            # Receiving from client
            method_start_time = time.time()

            data = ""

            while(True):
                new_data = conn.recv(8192)

                #print(new_data)

                #data = data + compression.decompress_string(new_data)

                data = data + new_data

                if(data.endswith(END_OF_MESSAGE)):
                    break

            if data == "":
                db.close()
                conn.close()
                return

            data = data[:len(END_OF_MESSAGE)*-1]

            data = data.decode("utf-8")

            print("Received: " + str(data))

            is_http = False
            is_resend = False

            error = False

            #Ist HTTP-Aufruf?
            try:
                data = parse_http_headers(data)
                is_http = True
            except AttributeError:
                try:
                    data = json.loads(data)
                    data = json.loads(data['msg'])
                    print("Request is not in HTTP-Format")
                except:
                    print("Data could not be parsed")
                    send_to_client(json.dumps({'status': 'error'}), conn, is_http)
                    error = True

            if 'resend' in data and data['resend'] is True:
                #Daten erneut senden
                print("Resend!")
                is_resend = True
                print(str(len(sent_data)) + " | " +sent_data)

                send_to_client(sent_data[data['resendstart']:], conn, is_http)

            if not error and not is_resend:
                if not is_resend:
                    message = process_data(data, db)

                    #print "compressing: "+message

                    #start_time = time.time()
                    #compressed_message = compression.compress_string(message)
                    #end_time = time.time()

                    #print "uncompressed: "+str(len(message))
                    #print "compressed: "+str(len(compressed_message))+", time: "+str(end_time-start_time)

                    #compressed_message = compressed_message.decode('iso-8859-1').encode('utf8')

                    msg = json.dumps({'msg': message, 'computing_time': (time.time() - method_start_time)*1000})

                if(msg is not None):
                    send_to_client(msg, conn, is_http)

                    sent_data = msg

                    save_request_to_db(data, msg, db)
                else:
                    print("Error in reply")
                    send_to_client(json.dumps({'status': 'error'}), conn, is_http)

                if not data:
                    break
        #except socket.error:
        #    break
        except Exception as e:
            traceback.print_exc()
            print("Fehler: "+str(e))
            send_to_client(json.dumps({'status': 'error'}), conn, is_http)

        #Stop endless loop
        break

    # came out of loop
    conn.close()
    db.commit()
    db.close()
    print("Verbindung beendet")

def listen_for_requests(s):
    #s = ssl.wrap_socket(s, keyfile=constants.LOCAL_SSL_PRIVKEY, certfile=constants.LOCAL_SSL_CERT, server_side=True)

    # wait to accept a connection - blocking call
    conn, addr = s.accept()
    print('Connected with ' + addr[0] + ':' + str(addr[1]))

    # start new thread takes 1st argument as a function name to be run, second is the tuple of arguments to the function.
    start_new_thread(clientthread, (conn,))

# now keep talking with the client

#Smart Speaker Communication
start_new_thread(smart_speaker_loop, ())

#Connect to MQTT-Broker
start_new_thread(mqtt_api.init_client,())

#Start Websocket-Server
#start_new_thread(start_websocket_server, ())

#Start HTTP-Server
#start_new_thread(webinterface.start_http_server, ())

#Start cronjobs
cronjobs.start_cronjobs()

#Communicate with cloud
start_new_thread(cloud_connection_loop, ())

#Lokale IP und Zertifikat regelmäßig aktualisieren
start_new_thread(update_ip_thread, ())
start_new_thread(update_cert_thread, ())

#Reset Image-Classifier flags
#db = get_database_con()
#set_server_data(ar_control.IS_TRAINING_TAG, "false", db)
#set_server_data(people_classifier.IS_TRAINING_TAG, "false", db)

#check if certs exist
check_cert(db=get_database_con())

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
print('Socket created')

# Bind socket to local host and port
try:
    s.bind((HOST, PORT))
except socket.error as msg:
    traceback.print_exc()
    print('Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1])
    sys.exit()

print('Socket bind complete')

# Start listening on socket
s.listen(10)
print('Socket now listening')

while True:
    try:
        listen_for_requests(s)
    except KeyboardInterrupt:
        break
    except:
        traceback.print_exc()

s.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-
import random
import urllib
import urllib2
import homevee.VoiceAssistant
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.voice_patterns import PATTERN_ACTIVITY


def get_pattern(db):
    return PATTERN_ACTIVITY

def get_label():
    return "activity"

def run_command(username, text, context, db):
    return get_activities(username, text, context, db)

def get_activities(username, text, context, db):
    try:
        url = helper.SMART_API_PATH + "/?action=activity&text=" + urllib.quote(text.encode('utf8'))
        print url

        response = urllib2.urlopen(url)
        data = response.read()

        if data is not None:
            return {'msg_speech': data, 'msg_text': data}
        else:
            output = get_error()
    except urllib2.HTTPError, e:
        output = get_error()

    return {'msg_speech': output, 'msg_text': output}

def get_error():
    return random.choice([
        'Mir fällt gerade nichts ein, was du tun könntest.'
    ])


#!/usr/bin/python
# -*- coding: utf-8 -*-

PHILIPS_HUE = "Philips Hue"
Z_WAVE = "Z-Wave"
FUNKSTECKDOSEN = "Funksteckdosen-Controller"
MAX_CUBE = "MAX! Cube"
MQTT_BROKER = "MQTT Broker"
MIYO_CUBE = "MIYO Cube"
RADEMACHER_HOMEPILOT = "Rademacher HomePilot"


#!/usr/bin/python
# -*- coding: utf-8 -*-

import time
import traceback

'''Gibt die Ereignisse zurück'''
def get_events(username, type, limit, offset, db):
    params = {'limit':limit, 'offset':offset}

    where_clause = ""

    if type is not None and type != "":
        params['type'] = type
        where_clause = "WHERE TYPE == :type "

    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM 'EVENTS' "+where_clause+"ORDER BY TIMESTAMP DESC LIMIT :limit OFFSET :offset", params)

        events = []

        for data in cur.fetchall():
            item = {'text': data['TEXT'], 'time': data['TIMESTAMP'], 'type': data['TYPE']}
            events.append(item)

        #EVENTS_LAST_CHECKED mit aktueller Zeit aktualisieren
        cur.execute("UPDATE 'userdata' SET 'EVENTS_LAST_CHECKED' = :time WHERE USERNAME == :username",
            {'time': time.time(), 'username': username})

        return {'events': events}

'''Gibt die Anzahl der ungesehenen Ereignisse zurück'''
def get_unseen_events(username, db):
    last_checked = None

    with db:
        try:
            cur = db.cursor()
            cur.execute("SELECT EVENTS_LAST_CHECKED FROM 'userdata' WHERE USERNAME == :username",
                {'username': username})

            data = cur.fetchone()

            if data is not None:
                last_checked = data['EVENTS_LAST_CHECKED']
        except Exception as e:
            traceback.print_exc()
            print(str(e))

        if last_checked is not None:
            cur.execute("SELECT COUNT(*) FROM 'EVENTS' WHERE TIMESTAMP > :time",
                        {'time': last_checked})

            for data in cur.fetchall():
                return {'eventcount': data['COUNT(*)']}
        else:
            return {'status': 'usernotfound'}

'''Gibt die vorhandenen Ereignis-Typen zurück'''
def get_event_types(db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT DISTINCT TYPE FROM EVENTS ORDER BY TYPE")

        types = []

        for data in cur.fetchall():
            types.append(data['TYPE'])

        return {'types': types}

'''Erstellt ein neues Ereignis'''
def add_event(type, text, db):
    with db:
        cur = db.cursor()
        cur.execute("INSERT INTO EVENTS (TYPE, TEXT, TIMESTAMP) VALUES (?,?,?)", [type, text, time.time()])
        return True


#!/usr/bin/python
# -*- coding: utf-8 -*-
import sqlite3

import os


def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d

'''Gibt eine Instanz der Datenbankverbindung zurück'''
def get_database_con():
    BASE_DIR = os.path.dirname(os.path.abspath(__file__))
    db_path = os.path.join(BASE_DIR, "data.db")

    #init db if not yet existing
    if not os.path.isfile(db_path):
        open(db_path, "w+")
        init_db()

    #print db_path

    con = sqlite3.connect(db_path)
    con.text_factory = str
    con.row_factory = dict_factory

    return con

def get_server_data(key, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT VALUE FROM SERVER_DATA WHERE KEY = :key",
                    {'key': key})

        try:
            return cur.fetchone()['VALUE']
        except Exception, e:
            return None

def set_server_data(key, value, db):
    with db:
        cur = db.cursor()

        cur.execute("INSERT OR IGNORE INTO SERVER_DATA (VALUE, KEY) VALUES(:value, :key)",
                    {'value': value, 'key': key})

        cur.execute("UPDATE OR IGNORE SERVER_DATA SET VALUE = :value WHERE KEY = :key",
                    {'value': value, 'key': key})

def init_db():
    command = """"""

    db = get_database_con()
    with db:
        cur = db.cursor()
        cur.execute(command)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json
from time import sleep

from homevee.Functions.condition_actions.actions import run_actions
from homevee.Functions.condition_actions.conditions import conditions_true
from homevee.db_utils import get_database_con


def run_trigger_automation(trigger_type, type, id, value, db):
    return

def run_timed_automations():
    #print "running automations"
    db = get_database_con()
    with db:
        cur = db.cursor()

        today = datetime.datetime.today()

        hour = today.hour
        minute = today.minute

        hour_string = str(hour)
        if(hour < 10):
            hour_string = "0"+hour_string

        minute_string = str(minute)
        if(minute < 10):
            minute_string = "0"+minute_string

        time_string = hour_string+":"+minute_string

        #print time_string

        cur.execute('SELECT * FROM AUTOMATION_DATA, AUTOMATION_TRIGGER_DATA WHERE AUTOMATION_DATA.ID = AUTOMATION_RULE_ID AND TYPE = "time" AND IS_ACTIVE = "true" AND VALUE = :val',
                    {'val': time_string})

        automations = cur.fetchall()

        run_automations(automations, db)


    return

def run_automations(automations, db):
    for item in automations:
        print item
        if (conditions_true(json.loads(item['CONDITION_DATA']), db)):
            if (int(item['TRIGGERED']) == 0):
                print "running item: "+str(item)
                run_actions(json.loads(item['ACTION_DATA']), db)
                set_triggered(item['ID'], 1, db)
        else:
            set_triggered(item['ID'], 0, db)

def set_triggered(id, triggered, db):
    with db:
        cur = db.cursor()

        cur.execute("UPDATE AUTOMATION_DATA SET TRIGGERED = :triggered WHERE ID = :id",
                    {'triggered': triggered, 'id': id})



def init_thread():
    #check = check_single_condition({'action': 'weekdays', 'days':[0]}, None)

    #if(check):
    #    print 'true'
    #else:
    #    print 'false'

    while(True):
        run_timed_automations()
        #print "waiting 1 minute"
        sleep(60)


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.Functions.tv_data import get_tv_plan
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.voice_patterns import PATTERN_TV_TIPPS

def get_pattern(db):
    return PATTERN_TV_TIPPS

def get_label():
    return "tvprogrammtipps"

def run_command(username, text, context, db):
    return get_tv_tipps(username, text, context, db)

def get_tv_tipps(username, text, context, db):
    tv_shows = get_tv_plan(username, "tipps", db)

    VERB = ['kommt', 'läuft']

    show_strings = []
    for show in tv_shows:
        TIME = show['time'] + ' Uhr'
        CHANNEL = show['channel']
        NAME = '\''+show['name']+'\''

        string_data = [
            ['um ', TIME, ' ', VERB, ' auf ', CHANNEL,  ' ', NAME],
            ['um ', TIME, ' ', VERB, ' ', NAME, ' auf ', CHANNEL],
            ['auf ', CHANNEL, ' um ', TIME, ' ', VERB, ' ', NAME],
            ['auf ', CHANNEL, ' ', VERB, ' ', NAME, ' um ', TIME],
            [NAME, ' ', VERB, ' um ', TIME, ' auf ', CHANNEL],
            [NAME, ' ', VERB, ' auf ', CHANNEL, ' um ', TIME],
        ]

        show_strings.append(helper.generate_string(string_data))

    show_string = None
    for i in range(0, len(show_strings)):
        if show_string is None:
            show_string = show_strings[i]
        elif i == len(show_strings)-1:
            if i != 0:
                show_string = show_string + ' und '
            show_string = show_string+show_strings[i]
        else:
            show_string = show_string+', '+show_strings[i]

    answer_data = [
        [['Die', 'Deine'], ' ', ['TV', 'Fernseh', 'Programm'], '-', ['Tipps', 'Vorschläge', 'Empfehlungen'], ' ',
            ['heute', 'für heute'], ' ', ['sind', 'lauten'], ':']
    ]

    output = helper.generate_string(answer_data) + ' ' + show_string + '.'

    return {'msg_speech': output, 'msg_text': output}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json

from homevee.Helper.helper_functions import has_permission, hash_password
from homevee.Functions.room_data import get_rooms
from homevee.Manager.places import get_place


def set_user_fcm_token(username, token, db):
    with db:
        cur = db.cursor()

        try:
            cur.execute("INSERT INTO PUSH_NOTIFICATION_TOKENS (USERNAME, TOKEN) VALUES (:user, :token)",
                {'token':token, 'user': username})
        except:
            return {'result': 'ok'}

    return {'result': 'ok'}

def get_users(username, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    with db:
        # Nutzer laden
        cur = db.cursor()
        cur.execute("SELECT * FROM USERDATA")

        users = []

        room_array = get_rooms(username, db)['rooms']

        for user in cur.fetchall():
            permission_array = json.loads(user['PERMISSIONS'])['permissions']

            permission_output = []

            if "admin" in permission_array:
                permission_output.append({'name': "Administrator", 'key': "admin"})

            for room in room_array:
                if str(room['location']) in permission_array:
                    permission_output.append({'name': room['name'], 'key': room['location']})

            if user['AT_HOME']:
                at_home = "true"
            else:
                at_home = "false"

            user_current_place = get_place(user['CURRENT_LOCATION'], db)

            user_location = None
            if(user_current_place is not None):
                user_location = user_current_place['NAME']

            user_item = {'username': user['USERNAME'], 'permissions': permission_output,
                'ip': user['IP'], 'at_home': at_home, 'location': user_location}

            users.append(user_item)

        cur.close()

        return {'userdata': users}

def delete_user(username, user_to_delete, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    with db:
        cur = db.cursor()
        cur.execute("DELETE FROM USERDATA WHERE USERNAME == :user'", {'user': user_to_delete})

        cur.close()

        #Bedingung anpassen
        if True:
            #delete_target_ids(username, user_to_delete, db)
            return {'status': 'ok'}
        else:
            return {'status': 'admin'}

def add_edit_user(username, name, psw, ip, permissions, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    hashed_pw, salt = hash_password(psw)

    param_array = {'username': name, 'ip': ip, 'permissions': permissions, 'password': hashed_pw, 'salt':salt}
    param_array2 = {'username': name, 'ip': ip, 'permissions': permissions}

    with db:
        cur = db.cursor()

        print psw

        if psw == "" or psw is None:
            print "not updating password"
            cur.execute("UPDATE OR IGNORE 'userdata' SET USERNAME = :username, IP = :ip, PERMISSIONS = :permissions WHERE USERNAME == :username;",
                    param_array2)
        else:
            print "updating password"
            cur.execute("UPDATE OR IGNORE 'userdata' SET USERNAME = :username, IP = :ip, PERMISSIONS = :permissions, PASSWORD = :password, PW_SALT = :salt WHERE USERNAME == :username;",
                param_array)

        cur.execute("INSERT OR IGNORE INTO 'userdata' (USERNAME, PASSWORD, PW_SALT, IP, PERMISSIONS) VALUES (:username, :password, :salt, :ip, :permissions);",
            param_array)

    return {'status': 'ok'}



    '''
    $paramArray = array('username' => $name, 'ip' => $ip, 'permissions' => $permissions, 'password' => password_hash($psw, 1));
	$paramArray2 = array('username' => $name, 'ip' => $ip, 'permissions' => $permissions);
	
	//Nutzer bearbeiten bzw. hinzufügen
    
	if($psw == null || $psw == ""){
		$updateQuery = $db->prepare("UPDATE OR IGNORE 'userdata'
							SET USERNAME = :username, IP = :ip, PERMISSIONS = :permissions
							WHERE USERNAME == :username;");
		$updateQuery->execute($paramArray2);
	}
	else{
		$updateQuery = $db->prepare("UPDATE OR IGNORE 'userdata'
							SET USERNAME = :username, PASSWORD = :password, IP = :ip, PERMISSIONS = :permissions
							WHERE USERNAME == :username;");
		$updateQuery->execute($paramArray);
	}
	
	$insertQuery = $db->prepare("INSERT OR IGNORE INTO 'userdata' (USERNAME, PASSWORD, IP, PERMISSIONS)
							VALUES (:username, :password, :ip, :permissions);");
	
	$insertQuery->execute($paramArray);
	
	if($updateQuery == true && $insertQuery == true) return "ok";
	else return "error";
    '''


#!/usr/bin/python
# -*- coding: utf-8 -*-
from thread import start_new_thread

import run_automations
import save_sensor_data
import save_energy_data
import user_ping
import video_surveillance
import zwave_value_loader
import philips_hue_value_loader
import weather_updater
import overview_notification
import calendar_reminder
import heating_scheme

def start_cronjobs():
    print "starting cronjobs..."

    # Load Z-Wave values
    start_new_thread(zwave_value_loader.init_thread, ())

    # Load Philips Hue values
    start_new_thread(philips_hue_value_loader.init_thread, ())

    # Save values to database
    start_new_thread(save_sensor_data.init_thread, ())

    # Save values to database
    start_new_thread(save_energy_data.init_thread, ())

    # Update weather cache
    start_new_thread(weather_updater.init_thread, ())

    # User anpingen
    # start_new_thread(cronjobs.user_ping.init_thread, ())

    # Benachrichtigung mit Infos für den Tag senden
    start_new_thread(overview_notification.init_thread, ())

    # Benachrichtigung mit Termin-Erinnerungen
    start_new_thread(calendar_reminder.init_thread, ())

    # Zeitgesteuerte Automatisierungen ausführen
    start_new_thread(run_automations.init_thread, ())

    # Heizplan ausführen
    start_new_thread(heating_scheme.init_thread, ())


#!/usr/bin/python
# -*- coding: utf-8 -*-

def get_shopping_list(username, db):
    items = []

    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM SHOPPING_LIST")

        for item in cur.fetchall():
            if item['AMOUNT'] is None:
                amount = -1
            else:
                amount = int(item['AMOUNT'])

            items.append({'id': item['ID'], 'amount': amount, 'item': item['ITEM']})

        cur.close()

        return {'items': items}

def add_edit_shopping_list_item(username, id, amount, name, db):
    #Bearbeiten oder Hinzufügen?

    with db:

        cur = db.cursor()

        if id is not "" and id is not None:
            #Add to list
            cur.execute("UPDATE OR IGNORE SHOPPING_LIST SET AMOUNT = :amount, ITEM = :name WHERE ID = :id",
                        {'amount': amount, 'name': name, 'id': id})
        else:
            #Edit list entry

            #Check if entry with same name exists and notify user(???)

            cur.execute("INSERT OR IGNORE INTO SHOPPING_LIST (AMOUNT, ITEM) VALUES (:amount, :name);",
                        {'amount': amount, 'name': name})

    #Existiert bereits? Menge addieren?

    return {'result': 'ok'}

def delete_shopping_list_item(username, id, db):
    with db:
        cur = db.cursor()
        cur.execute("DELETE FROM SHOPPING_LIST WHERE ID == :id",
            {'id': id})

        #Abfrage erfolgreich?
        if True:
            return {'result': 'ok'}
        else:
            return {'result': 'noadmin'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import traceback
import urllib2
from homevee.device_types import *
from homevee.DeviceAPI import zwave
from homevee.Helper.helper_functions import has_permission
from homevee.Manager.gateway import get_gateway


def set_modes(username, type, id, mode, db, check_user=True):
    if type == FUNKSTECKDOSE:
        return set_socket(username, id, mode, db, check_user)
    elif type == ZWAVE_SWITCH:
        return set_zwave_switch(username, id, mode, db, check_user)
    elif type == URL_SWITCH:
        return set_url_switch_binary(username, id, mode, db, check_user)
    elif type == URL_TOGGLE:
        return set_url_toggle(username, id, db, check_user)
    else:
        raise ValueError("Type does not exist")

def set_url_switch_binary(username, id, mode, db, check_user=True):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM URL_SWITCH_BINARY WHERE ID == :id",
                    {'id': id})

        data = cur.fetchone()

        if check_user and not has_permission(username, data['LOCATION'], db):
            return {'result': 'nopermission'}

        if mode == "":
            urllib2.urlopen(data['URL_ON'])
        else:
            urllib2.urlopen(data['URL_OFF'])

    return {'status': 'ok'}

def set_url_toggle(username, id, db, check_user=True):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM URL_TOGGLE WHERE ID == :id",
                    {'id': id})

        data = cur.fetchone()

        if check_user and not has_permission(username, data['LOCATION'], db):
            return {'result': 'nopermission'}

        urllib2.urlopen(data['TOGGLE_URL'])

    return {'status': 'ok'}

def set_socket(username, id, mode, db, check_user=True):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE == :device",
                    {'device': id})

        data = cur.fetchone()

        if check_user and not has_permission(username, data['ROOM'], db):
            return {'result': 'nopermission'}

        housecode = data['HAUSCODE']
        socketnum = data['STECKDOSENNUMMER']

        gateway = get_gateway('Funksteckdosen-Controller', db)
        print gateway

        if gateway is None or gateway['IP'] == 'localhost' or gateway['IP'] == '127.0.0.1':
            os.system("/usr/local/bin/send "+str(housecode)+" "+str(socketnum)+" "+str(mode))
        else:
            url = 'http://'+str(gateway['IP'])+'/funksteckdose.php?hauscode='+str(housecode)+\
                  '&steckdosennummer='+str(socketnum)+"&zustand="+str(mode)
            print url
            urllib2.urlopen(url)

        cur.execute("UPDATE FUNKSTECKDOSEN SET ZUSTAND = :zustand WHERE DEVICE == :device",
            {'zustand': mode, 'device': id})

        cur.close()

        return {'status': 'ok'}

def set_zwave_switch(username, id, mode, db, check_user=True):
    try:
        with db:
            cur = db.cursor()
            cur.execute("SELECT * FROM ZWAVE_SWITCHES WHERE ID == :device",
                        {'device': id})

            data = cur.fetchone()

            if check_user and not has_permission(username, data['LOCATION'], db):
                return {'result': 'nopermission'}

            device_id = data['ID']

            state = "on" if (int(mode) == 1) else "off"

            result = zwave.device_control.set_binary_device(device_id, state, db)

            if result['code'] == 200:
                cur.execute("UPDATE ZWAVE_SWITCHES SET VALUE = :value WHERE ID = :id",
                            {'value': mode, 'id': id})

                return {'result': 'ok'}
    except Exception, e:
        traceback.print_exc()
        return {'result': 'error'}

def set_diy_switch(id, mode, db, check_user=True):
    return


#!/usr/bin/python
# # -*- coding: utf-8 -*-
import datetime
from time import sleep

from homevee.db_utils import get_database_con
from homevee.device_types import *


def save_energy_data(db):
    with db:
        cur = db.cursor()

        date_now = datetime.datetime.now()
        date = date_now.strftime('%Y-%m-%d %H:%M')

        total_value = 0

        room_total_value = {}

        # Z-Wave Stromzähler
        cur.execute("SELECT * FROM ZWAVE_POWER_METER")
        for item in cur.fetchall():
            room_id = item['ROOM_ID']
            value = save_to_db(date_now, ZWAVE_POWER_METER, room_id, item['DEVICE_ID'],
                               item['VALUE'], db)
            total_value += value

            if room_id in room_total_value:
                room_total_value[room_id] += value
            else:
                room_total_value[room_id] = value

        #Gesamtwerte der Räume speichern
        for room in room_total_value:
            print "inserting in room_energy_data of room: "+str(room)
            cur.execute("INSERT INTO ROOM_ENERGY_DATA (ROOM_ID, POWER_USAGE, DATE) VALUES (:room, :usage, :date)",
                        {'room': room, 'usage': room_total_value[room_id], 'date': date})

        #Gesamtverbrauch speichern
        print "inserting in energy_data"
        cur.execute("INSERT INTO ENERGY_DATA (DATE, POWER_USAGE) VALUES (:date, :value)",
                    {'date': date, 'value': total_value})

def save_to_db(date_now, type, room_id, id, value, db):
    value = float(value)

    with db:
        cur = db.cursor()
        date = date_now.strftime('%Y-%m-%d %H:%M')

        if type == ZWAVE_POWER_METER:
            cur.execute("SELECT PREV_VALUE FROM ZWAVE_POWER_METER WHERE DEVICE_ID = :id", {'id': id})
            result = cur.fetchone()
            prev_value = float(result['PREV_VALUE'])

            prev_value = prev_value+value
            print "prev_value: "+str(prev_value)
            print "updating zwave_power_meter value"
            cur.execute("UPDATE ZWAVE_POWER_METER SET PREV_VALUE = :val, VALUE = 0 WHERE DEVICE_ID = :id",
                {'val': prev_value, 'id': id})

        print "inserting in device_energy_data"
        cur.execute("INSERT INTO DEVICE_ENERGY_DATA (LOCATION, DEVICE_ID, DEVICE_TYPE, DATE, POWER_USAGE) \
                        VALUES (:location, :id, :type, :date, :value)",
                    {'location': room_id, 'id': id, 'type': type, 'date': date, 'value': value})

        return value

def init_thread():
    print "energy saving thread initiated"
    while True:
        db = get_database_con()

        t = datetime.datetime.today()

        seconds_to_wait = (24 * 60 * 60) - (t.hour * 60 * 60) - (t.minute * 60) - t.second - 60 #um 23:59 ausführen
        #seconds_to_wait = 5
        print "waiting for "+str(seconds_to_wait)+" seconds"
        sleep(seconds_to_wait)
        print("Saving Energy-data")
        save_energy_data(db)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.DeviceAPI.zwave.get_devices import get_device_value
from homevee.device_types import *
from homevee.Helper.permissions import has_permission

SENSOR_TYPE_MAP = {
	'temp': {'name': 'Temperatur', 'einheit': '°C', 'einheit_word': 'Grad'},
	'hygro': {'name': 'Luftfeuchtigkeit', 'einheit': '%', 'einheit_word': '%'},
	'helligkeit': {'name': 'Helligkeit', 'einheit': 'Lux', 'einheit_word': 'Lumen'},
	'uv': {'name': 'UV-Licht', 'einheit': 'UV-Index', 'einheit_word': 'UV-Index'},
	'powermeter': {'name': 'Stromverbrauch', 'einheit': 'Watt', 'einheit_word': 'Watt'},
}

#def get_sensor_types():
#    return SENSOR_TYPE_MAP

def get_einheit(type, id, db):
	with db:
		cur = db.cursor()
		db_data = {
			ZWAVE_SENSOR: {'table': 'ZWAVE_SENSOREN', 'sensor_type_col': 'SENSOR_TYPE', 'id_col': 'ID'},
			MQTT_SENSOR: {'table': 'MQTT_SENSORS', 'sensor_type_col': 'TYPE', 'id_col': 'ID'}
		}

		cur.execute("SELECT "+db_data[type]['sensor_type_col']+" FROM "+db_data[type]['table']+" WHERE "+db_data[type]['id_col']+" = :id",
					{'id': id})

		return SENSOR_TYPE_MAP[cur.fetchone()[db_data[type]['sensor_type_col']]]['einheit']

def get_sensor_value(type, id, show_einheit, db):
	if type == ZWAVE_SENSOR:
		if id is not None:
			value = get_device_value(type, id, db)

			try:
				output = round(float(value), 1)

				if show_einheit:
					output = str(output) + " " + get_einheit(type, id, db)
			except:
				output = "N/A"

			return output
	elif type == MQTT_SENSOR:
		if id is not None:
			with db:
				cur = db.cursor()
				cur.execute("SELECT * FROM MQTT_SENSORS WHERE ID == :id",
							{'id': id})

				data = cur.fetchone()

				value = data['LAST_VALUE']

				try:
					output = round(float(value), 1)
					if show_einheit:
						output = str(output) + " " + get_einheit(type, id, db)
				except:
					output = "N/A"

				return output
	# Andere Typen hinzufügen

def get_sensor_data(username, room, type, id, show_einheit, db):
	if not has_permission(username, room, db):
		return {'status': 'nopermission'}

	if type is not None and type != "" and id is not None and id != "":
		return get_sensor_value(type, id, show_einheit, db)
	else:
		with db:
			cur = db.cursor()

			#Räume laden
			if room == "all":
				cur.execute("SELECT * FROM ROOMS")
			else:
				cur.execute("SELECT * FROM ROOMS WHERE LOCATION == :room",
					{'room': room})

			values = []

			for room in cur.fetchall():
				value_array = []

				if not has_permission(username, room['LOCATION'], db):
					continue
					#return "nopermission"

				#Alle Z-Wave Sensoren
				type = ZWAVE_SENSOR
				cur.execute("SELECT * FROM ZWAVE_SENSOREN WHERE RAUM == :location",{'location': room['LOCATION']})

				for sensor in cur.fetchall():
					value = {'shortform': sensor['SHORTFORM'], 'id': sensor['ID'], 'device_type': type, 'icon': sensor['ICON'],
						'wert': get_sensor_data(username, room['LOCATION'], type, sensor['ID'], 1, db)}
					value_array.append(value)

				#Alle MQTT Sensoren
				type = MQTT_SENSOR
				cur.execute("SELECT * FROM MQTT_SENSORS WHERE ROOM == :location",{'location': room['LOCATION']})

				for sensor in cur.fetchall():
					value = {'shortform': sensor['NAME'], 'id': sensor['ID'], 'device_type': type, 'icon': sensor['ICON'],
						'wert': get_sensor_data(username, room['LOCATION'], type, sensor['ID'], 1, db)}
					value_array.append(value)

				#Alle anderen Sensoren

				value_item = {'name': room['NAME'], 'location': room['LOCATION'], 'icon': room['ICON'], 'value_array': value_array}

				values.append(value_item)

			cur.close()

			return {'values': values}




#!/usr/bin/python
# -*- coding: utf-8 -*-
from time import sleep

from homevee.DeviceAPI.zwave.get_devices import set_device_value
from homevee.DeviceAPI.zwave.utils import do_zwave_request
from homevee.db_utils import get_database_con
from homevee.device_types import *

def load_zwave_values(db):
    with db:
        cur = db.cursor()

        # Alle Z-Wave Gerätetypen durchlaufen und den aktuellen Wert in Datenbank schreiben

        #Stromzähler
        TYPE = ZWAVE_POWER_METER
        cur.execute("SELECT * FROM ZWAVE_POWER_METER")
        data = cur.fetchall()

        for item in data:
            ID = item['DEVICE_ID']
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID, db)

            if result['code'] != 200:
                value = "N/A"
            else:
                value = result['data']['metrics']['level']

            set_device_value(TYPE, ID, value, db)

        #Thermostat
        TYPE = ZWAVE_THERMOSTAT
        cur.execute("SELECT * FROM ZWAVE_THERMOSTATS")
        data = cur.fetchall()

        for item in data:
            ID = item['THERMOSTAT_ID']
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID, db)

            if result['code'] != 200:
                value = "N/A"
            else:
                value = result['data']['metrics']['level']

            set_device_value(TYPE, ID, value, db)

        #Sensor
        TYPE = ZWAVE_SENSOR
        cur.execute("SELECT * FROM ZWAVE_SENSOREN")
        data = cur.fetchall()

        for item in data:
            ID = item['ID']
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID, db)

            if result['code'] != 200:
                value = "N/A"
            else:
                value = result['data']['metrics']['level']

                #if item['VALUE'] != value:
                    #trigger automation

            set_device_value(TYPE, ID, value, db)

        #Schalter
        TYPE = ZWAVE_SWITCH
        cur.execute("SELECT * FROM ZWAVE_SWITCHES")
        data = cur.fetchall()

        for item in data:
            ID = item['ID']
            result = do_zwave_request("/ZAutomation/api/v1/devices/" + ID, db)

            if result['code'] != 200:
                value = "N/A"
            else:
                value = result['data']['metrics']['level']

                value = value == "on"

            set_device_value(TYPE, ID, value, db)

def init_thread():
    while True:
        db = get_database_con()
        load_zwave_values(db)
        db.close()
        sleep(5*60)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json
import random

from homevee.VoiceAssistant import helper, context_keys
from homevee.VoiceAssistant.voice_patterns import PATTERN_WEATHER
from homevee.db_utils import get_server_data

def get_pattern(db):
    return PATTERN_WEATHER

def get_label():
    return "weather"

def run_command(username, text, context, db):
    return get_weather(username, text, context, db)

def get_weather(username, text, context, db):
    weather = get_server_data('WEATHER_CACHE', db)
    weather = json.loads(weather)
    city_name = weather['city']['name']
    weather_list = weather['list']

    now = datetime.datetime.now()

    cur_weekday = now.weekday()

    search_weekday_word, search_weekday = get_weekday(text, cur_weekday)

    if (context is not None and search_weekday is None):
        return False

    if search_weekday is None:
        #No day given => output tdays weather
        search_weekday_word = "Heute"
        search_weekday = cur_weekday

    search_daytime = get_day_time(text)

    #current day after search day?
    if search_weekday < cur_weekday:
        #add one week to search day, since its next week
        search_weekday += 7

    day_difference = search_weekday - cur_weekday

    #add the difference
    search_date = now + datetime.timedelta(days=day_difference)

    output = None

    for day in weather_list:
        cur_date = datetime.datetime.fromtimestamp(day['dt'])
        if (cur_date.date() == search_date.date()):
            single_time = False
            single_time_string = None
            if search_daytime is not None:
                single_time = True
                temp = int(day['temp'][search_daytime])

                data = [
                    ['bei ' ,['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' ', temp, ' Grad'],
                    ['und die Temperatur liegt bei ' ,['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' ', temp, ' Grad'],
                    ['und die Temperatur liegt ', ['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' bei ', ' ', temp, ' Grad'],
                    ['und es wird ' ,['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' ', temp, ' Grad haben'],
                    ['und es hat ' ,['ungefähr', 'in etwa', 'etwa', 'ca.', ''], ' ', temp, ' Grad']
                ]
                temp_string = helper.generate_string(data)

                time_strings = {
                    'morn': ['früh', 'in der Früh'],
                    'day': ['tagsüber'],
                    'eve': ['Abend', 'am Abend'],
                    'night': ['in der Nacht', 'nachts']
                }
                single_time_string = random.choice(time_strings[search_daytime])
            else:
                temp = {'min': int(day['temp']['min']), 'max': int(day['temp']['max'])}

                if(temp['min'] == temp['max']):
                    data = [
                        ['und die Temperatur ', ['beträgt', 'liegt bei'], ' '+str(temp['max'])+' Grad'],
                        ['bei '+str(temp['min'])+' Grad'],
                        ['bei einer Temperatur von '+str(temp['max'])+' Grad']
                    ]
                else:
                    data = [
                        ['und die Temperaturen ', ['betragen', 'liegen'], ' zwischen '+str(temp['min'])+' und '+str(temp['max'])+' Grad'],
                        ['bei '+str(temp['min'])+' bis '+str(temp['max'])+' Grad'],
                        ['bei Temperaturen zwischen '+str(temp['min'])+' und '+str(temp['max'])+' Grad']
                    ]
                temp_string = helper.generate_string(data)

            weather_ids = []

            for weather in day['weather']:
                weather_ids.append(weather['id'])

            weather_descs = get_weather_description(weather_ids)

            weather_descs_string = None
            for i in range(0, len(weather_descs)):
                if weather_descs_string is None:
                    weather_descs_string = weather_descs[i]
                elif i == len(weather_descs)-1:
                    weather_descs_string = ' und '+weather_descs[i]
                else:
                    weather_descs_string = ', '+weather_descs[i]

            if day_difference <= 2:
                day_desc_words = ['Heute', 'Morgen', 'Übermorgen']
                day_string = day_desc_words[day_difference]

                if single_time_string is not None:
                    day_string += ' '+single_time_string

            else:
                if single_time_string is None:
                    day_string = helper.generate_string([
                        ['Am '+search_weekday_word],
                        [search_weekday_word+'s']
                    ])
                else:
                    day_string = search_weekday_word+' '+single_time_string

            output_data = [
                [day_string + ' ', ['gibt', 'hat'] ,' es ' + weather_descs_string + ' ' + temp_string + '.']
            ]
            output = helper.generate_string(output_data)

            print "set context"
            helper.set_context(username, context_keys.GET_WEATHER, {}, db)

    return {'msg_speech': output, 'msg_text': output}

def get_weekday(text, cur_weekday):
    words = text.split(' ')

    #normal weekday
    days = ['montag', 'dienstag', 'mittwoch', 'donnerstag', 'freitag', 'samstag', 'sonntag']
    for i in range(0, len(days)):
        day = days[i]
        if day in words:
            return day.capitalize(), i

    day_words = ['heute', 'morgen', 'übermorgen']
    for i in range(0, len(day_words)):
        day = day_words[i]
        if day in words:
            output_day = cur_weekday + i

            if output_day >= 7:
                #subtract one week
                output_day -= 7

            return day.capitalize(), output_day

    return None, None

def get_day_time(text):
    day_times = [
        {'time': 'morn', 'keywords': ['morgens', 'früh']},
        {'time': 'day', 'keywords': ['tagsüber']},
        {'time': 'eve', 'keywords': ['abends', 'abend']},
        {'time': 'night', 'keywords': ['nachts', 'nacht']}
    ]

    words = text.split(' ')

    for day_time in day_times:
        for keyword in day_time['keywords']:
            if keyword in words:
                return day_time['time']

    return None

def get_weather_description(codes):
    description_map = {
        #Gewitter
        200: 'Gewitter mit leichtem Regen',
        201: 'Gewitter mit Regen',
        202: 'Gewitter mit starkem Regen',
        210: 'leichte Gewitter',
        211: 'Gewitter',
        212: 'starke Gewitter',
        221: 'Gewitter mit leichtem Regen',
        230: 'Gewitter mit leichtem Regen',
        231: 'Gewitter mit leichtem Regen',
        232: 'Gewitter mit leichtem Regen',

        #Nieselregen
        300: 'Nieselregen',
        301: 'Nieselregen',
        302: 'Nieselregen',
        310: 'Nieselregen',
        311: 'Nieselregen',
        312: 'Nieselregen',
        313: 'Nieselregen',
        314: 'Nieselregen',
        321: 'Nieselregen',

        #Regen
        500: 'leichten Regen',
        501: 'mittelstarken Regen',
        502: 'starken Regen',
        503: 'sehr starken Regen',
        504: 'extremen Regen',
        511: 'gefrierenden Regen',
        520: 'leichte Schauer',
        521: 'Schauer',
        522: 'starke Schauer',
        531: 'starke Schauer',

        #Schnee
        600: 'leichten Schneefall',
        601: 'Schneefall',
        602: 'starken Schneefall',
        611: 'Schneeregen',
        612: 'Schneeregen-Schauer',
        615: 'leichten Regen und Schnee',
        616: 'Regen und Schnee',
        620: 'leichte Schneeschauer',
        621: 'Schneeschauer',
        622: 'starke Schneeschauer',

        #Atmosphäre
        701: 'Nebel',
        711: 'Rauch',
        721: 'Dunst',
        731: 'Staubwirbel',
        741: 'Nebel',
        751: 'Sand',
        761: 'Staub',
        762: 'Vulkanasche',
        771: 'Böen',
        781: 'Tornado',

        #Klar
        800: 'klaren Himmel',

        #Wolken
        801: 'wenige Wolken',
        802: 'aufgelockerte Wolken',
        803: 'aufgelockerte Wolken',
        804: 'bedeckte Bewölkung'
    }

    output = []

    for code in codes:
        if code in description_map:
            output.append(description_map[code])

    return output


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.DeviceAPI.wake_on_lan import wake_on_lan
from homevee.VoiceAssistant import context_keys
from homevee.VoiceAssistant.helper import generate_string, set_context, find_room, get_okay
from homevee.device_types import *


def voice_wol(username, text, context, db):
    room = find_room(text, db)

    if room is not None:
        room_key = room['LOCATION']
    else:
        room_key = None

    devices = find_devices(text, room_key, db)

    if len(devices) == 0:
        #Keine Geräte gefunden
        answer_data = [
            [['Dieses ', 'Das genannte '], 'Gerät ', ['existiert nicht.', 'gibt es nicht.', 'wurde noch nicht angelegt.']]
        ]

        answer = generate_string(answer_data)

        return {'msg_speech': answer, 'msg_text': answer}

    words = text.split(" ")

    #Geräte schalten
    for device in devices:
        if device['type'] == XBOX_ONE_WOL:
            print ""
            #xbox_wake_up(username, device['id'], db)
        else:
            wake_on_lan(username, device['id'], db)

    set_context(username, context_keys.WOL, {'location': room, 'devices': devices}, db)

    DEVICE_STRING = None
    if len(devices) > 1:
        VERB = "wurden"
        DEVICE_WORD = "Die Geräte "

        for i in range(0, len(devices)):
            if DEVICE_STRING is None:
                DEVICE_STRING = '\'' + devices[i]['name'] + '\''
            elif i == len(devices) - 1:
                DEVICE_STRING = DEVICE_STRING + ' und ' + '\'' + devices[i]['name'] + '\''
            else:
                DEVICE_STRING = DEVICE_STRING + ', ' + '\'' + devices[i]['name'] + '\''
    else:
        DEVICE_WORD = "Das Gerät "
        VERB = "wurde"

        DEVICE_STRING = '\'' + devices[0]['name'] + '\''

    answer_data = [
        [get_okay(), [', ' + username, ''], '.',
         ['', [' ', DEVICE_WORD,  [DEVICE_STRING + ' ', ''], VERB + ' ',
               ['gestartet', 'eingeschalten', 'eingeschaltet', 'an gemacht', 'hochgefahren'], '.']]]
    ]

    answer = generate_string(answer_data)

    return {'msg_speech': answer, 'msg_text': answer}

def find_devices(text, location_key, db):
    devices = []

    with db:
        cur = db.cursor()

        #data => table_name, name_key, id_key, type_key, location_key
        data = [
            ['WAKE_ON_LAN','NAME','DEVICE', WAKE_ON_LAN,'LOCATION'],
            ['XBOX_ONE_WOL','NAME','ID', XBOX_ONE_WOL,'LOCATION']
        ]

        for item in data:
            if location_key is None:
                cur.execute("SELECT * FROM "+item[0])
            else:
                param_array = {'location': location_key}
                cur.execute("SELECT * FROM "+item[0]+" WHERE "+item[4]+" = :location", param_array)

            for device in cur.fetchall():
                position = text.find(device[item[1]].lower())
                if position is not -1:
                    device_item = {'type': item[3], 'name': device[item[1]], 'id': device[item[2]]}
                    devices.append(device_item)

    return devices


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.Functions.room_data import get_room_data
from homevee.Helper.helper_functions import has_permission
from homevee.Manager.scenes import get_scenes


def add_edit_room(username, room_name, room_key, icon, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    param_array = {'name': room_name, 'icon': icon}

    with db:
        cur = db.cursor()

        if room_key is not None and room_key != "":
            #print "edit room"
            param_array['location'] = room_key
            cur.execute("UPDATE ROOMS SET NAME = :name, ICON = :icon WHERE LOCATION == :location;",
                param_array)
        else:
            #print "add room"
            cur.execute("INSERT INTO ROOMS (NAME, ICON) VALUES (:name, :icon);",
                    param_array)

    return {'result': 'ok'}

def delete_room(username, room_key, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    roomdata = get_room_data(username, room_key, db)['roomdata']

    if len(roomdata) == 1:
        if roomdata[0]['type'] == "scenes":
            scenes = get_scenes(username, room_key, db)

            for scene in scenes['scenes']:
                if scene['room'] == room_key:
                    return "roomhasitems"
    elif len(roomdata) > 1:
        return "roomhasitems"

    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM ROOMS WHERE LOCATION == :location",
                    {'location': room_key})

    return {'result': 'ok'}

def move_items_and_delete_old_room(username, old_room, new_room, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    params = {'oldroom': old_room, 'newroom': new_room}

    query_array = []

    #Funksteckdosen
    query_array.append("UPDATE 'funksteckdosen' SET ROOM = :newroom WHERE ROOM == :oldroom;")

    #Z-Wave
    query_array.append("UPDATE 'ZWAVE_SENSOREN' SET RAUM = :newroom WHERE RAUM == :oldroom;")
    query_array.append("UPDATE 'ZWAVE_THERMOSTATS' SET RAUM = :newroom WHERE RAUM == :oldroom;")

    #DIY
    query_array.append("UPDATE 'DIY_DEVICES' SET ROOM = :newroom WHERE ROOM == :oldroom;")
    query_array.append("UPDATE 'DIY_SENSORS' SET RAUM = :newroom WHERE RAUM == :oldroom;")
    query_array.append("UPDATE 'DIY_REEDSENSORS' SET RAUM = :newroom WHERE RAUM == :oldroom;")
    query_array.append("UPDATE 'DIY_SWITCHES' SET RAUM = :newroom WHERE RAUM == :oldroom;")

    #Szenen
    query_array.append("UPDATE 'SCENES' SET ROOM = :newroom WHERE ROOM == :oldroom;")

    with db:
        cur = db.cursor()
        for query in query_array:
            cur.execute(query, params)

    return delete_room(username, old_room, db)

def delete_room_with_items(username, room_key, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    params = {'oldroom': room_key}

    query_array = []

    # Funksteckdosen
    query_array.append("DELETE FROM 'funksteckdosen' WHERE ROOM == :oldroom;")

    # Z-Wave
    query_array.append("DELETE FROM 'ZWAVE_SENSOREN' WHERE RAUM == :oldroom;")
    query_array.append("DELETE FROM 'ZWAVE_THERMOSTATS' WHERE RAUM == :oldroom;")

    # DIY
    query_array.append("DELETE FROM 'DIY_DEVICES' WHERE ROOM == :oldroom;")
    query_array.append("DELETE FROM 'DIY_REEDSENSORS' WHERE RAUM == :oldroom;")
    query_array.append("DELETE FROM 'DIY_SENSORS' WHERE RAUM == :oldroom;")
    query_array.append("DELETE FROM 'DIY_SWITCHES' WHERE RAUM == :oldroom;")

    # Szenen
    query_array.append("DELETE FROM 'SCENES' WHERE ROOM == :oldroom;")

    with db:
        cur = db.cursor()
        for query in query_array:
            cur.execute(query, params)

    return delete_room(username, room_key, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import socket
import subprocess
import time
import traceback
import urllib
from OpenSSL import crypto
from socket import gethostname

import os
import urllib2

from passlib.hash import pbkdf2_sha512
#from Helper.HTTPRequest import HTTPRequest
from homevee import constants
#from Helper import compression
from homevee.db_utils import get_database_con, set_server_data, get_server_data

'''Prüft, ob der Nutzer die angegebene Berechtigung besitzt'''
def has_permission(username, permission, db):
    permissions = get_permissions(username, db)

    if permissions is None:
        return False
    else:
        return (permission in permissions or 'admin' in permissions)

def get_permissions(username, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT PERMISSIONS FROM USERDATA WHERE USERNAME = :user", {'user': username})

        result = cur.fetchone()

        if result is not None:
            permissions = json.loads(result['PERMISSIONS'])['permissions']

            return permissions
        else:
            return None

def save_request_to_db(data, reply, db):
    return

    if 'password' in data:
        del data['password']

    if data['action'] == "arcontrol":
        del data['imagedata']

    with db:
        cur = db.cursor()

        cur.execute("INSERT INTO REQUEST_DATA (RESPONSE) VALUES (:response)",
            {'response': reply})

        request_id = cur.lastrowid

        for key in data.keys():
            cur.execute("INSERT INTO REQUEST_DATA_PARAMS (REQUEST_ID, PARAM_KEY, PARAM_VALUE) VALUES(:id, :key, :value)",
                {'id': request_id, 'key': key, 'value': data[key]})

        cur.close();

        return True

def send_to_client(data, conn, is_http):
    if not is_http:
        print("Sent Response: "+data)
    elif is_http:
        data = 'HTTP/1.1 200 OK\nContent-Type: text/html\n'+data+'\n'
        print("Sent HTTP-Response: "+data)

    #Prüfen, ob alle Daten gesendet wurden
    len_send = conn.send(bytearray(str.encode(data)))
    #len_send = conn.send(compressed_data)
    print("Data: "+str(len(data))+" | Sent: "+str(len_send))

    #if(len_send is 0):
    #    send_to_client(json.dumps({'status': 'error'}), conn, is_http)

def get_rooms(username, db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM ROOMS")

        rooms = []

        for data in cur.fetchall():
            if has_permission(username, data['LOCATION'], db):
                rooms.append({'name': data['NAME'], 'location': data['LOCATION'], 'icon': data['ICON']})

        cur.close()

        return {'rooms': rooms}

def parse_http_headers(data):
    if data.startswith("GET"):
        print("HTTP-GET-Request")
        #request = HTTPRequest(data).path
        #path, param_string = request.split('?', 1)
    elif data.startswith("POST"):
        print("HTTP-POST-Request")
        lines = data.split('\n')
        param_string = lines[-1]
    else:
        raise AttributeError

    params = {}

    param_fragments = param_string.split('&')

    for param in param_fragments:
        if '=' in param:
            key, value = param.split('=')
            params[key] = value

    return params

def hash_password(password):
    salt = os.urandom(12).encode('hex')

    hashed_pw = pbkdf2_sha512.encrypt(password+salt, rounds=200000)

    #print "Password: "+password
    #print "Salt: "+salt
    #print "Hashed: "+hashed_pw

    return (hashed_pw, salt)

def verify_password(password, salt, hashed_pw):
    return pbkdf2_sha512.verify(password+salt, hashed_pw)

def verify_user(username, password):
    db = get_database_con()

    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM USERDATA WHERE USERNAME == :user", {'user': username})

        user = cur.fetchone()

        if(user is None):
            return False

        hashed_pw = user['PASSWORD']
        salt = user['PW_SALT']

        verified = verify_password(password, salt, hashed_pw)

        #print "Username: "+username
        #print "Password: " + password
        #print "Salt: " + salt
        #print "Hashed: " + hashed_pw
        #print "Verified: "+str(verified)

        return verified

def remote_control_enabled(db):
    REMOTE_CONTROL_ENABLED = get_server_data("USE_REMOTE_CONTROL", db)
    return REMOTE_CONTROL_ENABLED == "true"

def set_remote_control_enabled(username, enabled, db):
    if(not has_permission(username, "admin", db)):
        return {'result': 'noadmin'}
    set_server_data("USE_REMOTE_CONTROL", enabled, db)

    return {'result': 'ok'}

def save_remote_data(username, remote_id, linked_account, db):
    if(not has_permission(username, "admin", db)):
        return {'result': 'noadmin'}

    set_server_data("REMOTE_ID", remote_id, db)
    set_server_data("REMOTE_LINKED_ACCOUNT", linked_account, db)

    return {'result': 'ok'}

def load_remote_data(username, db):
    if(not has_permission(username, "admin", db)):
        return {'result': 'noadmin'}

    remote_id = get_server_data("REMOTE_ID", db)
    access_token = get_server_data("REMOTE_ACCESS_TOKEN", db)

    linked_account = get_server_data("REMOTE_LINKED_ACCOUNT", db)

    #check if account is still linked

    #load premium membership
    url = "https://cloud.homevee.de/server-api.php?action=getpremiumuntil&remoteid="+remote_id+"&accesstoken="+access_token
    contents = urllib2.urlopen(url).read()
    data = json.loads(contents)
    is_premium = None

    try:
        is_premium = data['is_premium']
        premium_until = data['premium_until']
    except:
        premium_until = None

    return {'remote_id': remote_id,
            'access_token': access_token,
            'linked_account': linked_account, 'premium_until': premium_until, 'is_premium': is_premium,
            'remote_control_enabled': remote_control_enabled(db)}

def connect_remote_id_with_account(username, account_name, account_secret, db):
    if(not has_permission(username, "admin", db)):
        return {'result': 'noadmin'}

    remote_id = get_server_data("REMOTE_ID", db)
    url = "https://homevee.de/connect-remote-id.php?remote_id="+remote_id+"&account_name="+account_name+"&account_secret="+account_secret

    response = urllib.urlopen(url).read()

    if response == "ok":
        set_server_data("REMOTE_LINKED_ACCOUNT", account_name, db)

    return {'result': response}

def update_ip_thread():
    last_ip = None

    db = get_database_con()

    while(True):
        remote_id = get_server_data("REMOTE_ID", db)
        access_token = get_server_data("REMOTE_ACCESS_TOKEN", db)

        my_ip = get_my_ip()

        if my_ip != last_ip or my_ip != get_my_ip_from_cloud(remote_id):
            update_ip(my_ip, remote_id, access_token)

        last_ip = my_ip

        #wait some time
        time.sleep(5*60) #5 Minuten

def update_cert_thread():
    db = get_database_con()

    while (True):
        remote_id = get_server_data("REMOTE_ID", db)
        access_token = get_server_data("REMOTE_ACCESS_TOKEN", db)

        # generate_cert

        check_cert(None, remote_id, access_token)

        # wait some time
        time.sleep(12 *60 * 60)  # 12 Stunden

def check_cert(db=None, remote_id=None, access_token=None):
    cert_data = get_local_cert()

    if db is not None:
        remote_id = get_server_data("REMOTE_ID", db)
        access_token = get_server_data("REMOTE_ACCESS_TOKEN", db)

    if cert_data is None:
        generate_cert()

    if cert_data != get_my_cert_from_cloud(remote_id):
        update_cert(cert_data, remote_id, access_token)

def update_ip(ip, remote_id, access_token):

    MAX_RETRIES = 10
    retries = 0

    contents = None

    # update local ip in cloud
    url = "https://cloud.homevee.de/server-api.php?action=updatelocalip&remoteid=" + remote_id + "&accesstoken=" + access_token + "&localip=" + ip
    while(contents != "ok" and retries < MAX_RETRIES):
        try:
            contents = urllib2.urlopen(url).read()
        except:
            traceback.print_exc()

        retries += 1

def update_cert(cert_data, remote_id, access_token):
    print "updating local cert..."

    cert_data = cert_data.replace("\n", "")
    cert_data = cert_data.replace("-----BEGIN CERTIFICATE-----", "")
    cert_data = cert_data.replace("-----END CERTIFICATE-----", "")

    MAX_RETRIES = 10
    retries = 0

    contents = None

    # update cert in cloud
    url = "https://cloud.homevee.de/server-api.php?action=updatecert&remoteid=" + remote_id + "&accesstoken=" + access_token + "&cert=" + urllib.quote(
        cert_data)
    print url

    while (contents != "ok" and retries < MAX_RETRIES):
        try:
            contents = urllib2.urlopen(url).read()
        except:
            traceback.print_exc()

        retries += 1

    print contents

def get_my_ip():
    #return "192.168.2.110"

    try:
        cmd = "hostname -i"
        data = subprocess.check_output(cmd, shell=True).decode('utf-8')
        #return "192.168.2.110"
        ip, mac = data.split(" ")
    except:
        ip = socket.gethostbyname(socket.gethostname())

    print "my ip address: " + ip

    return ip

def generate_cert():
    # create a key pair
    k = crypto.PKey()
    k.generate_key(crypto.TYPE_RSA, 1024)

    # create a self-signed cert
    cert = crypto.X509()
    cert.get_subject().C = "DE"
    cert.get_subject().ST = "Germany"
    cert.get_subject().L = "Germany"
    cert.get_subject().O = "Homevee"
    cert.get_subject().OU = "Homevee"
    cert.get_subject().CN = gethostname()
    cert.set_serial_number(1000)
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(10 * 365 * 24 * 60 * 60)
    cert.set_issuer(cert.get_subject())
    cert.set_pubkey(k)
    cert.sign(k, 'sha1')

    open(constants.LOCAL_SSL_CERT, "wt").write(
        crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
    open(constants.LOCAL_SSL_PRIVKEY, "wt").write(
        crypto.dump_privatekey(crypto.FILETYPE_PEM, k))

def get_local_cert():
    if(os.path.isfile(constants.LOCAL_SSL_CERT)):
        file = open(constants.LOCAL_SSL_CERT, "r")
        content = file.read()
        if content is not None and content != '':
            return content

    return None

def get_my_ip_from_cloud(remote_id):
    try:
        url = "http://cloud.homevee.de/server-api.php?action=getlocalip&remoteid="+remote_id
        contents = urllib2.urlopen(url).read()

        data = json.loads(contents)

        if 'ip' in data and data['ip'] is not None:
            return data['ip']
        else:
            return None
    except:
        traceback.print_exc()
        return None

def get_my_cert_from_cloud(remote_id):
    try:
        url = "http://cloud.homevee.de/server-api.php?action=getcert&remoteid="+remote_id
        contents = urllib2.urlopen(url).read()

        data = json.loads(contents)

        if 'cert' in data and data['cert'] is not None:
            return data['cert']
        else:
            return None
    except:
        traceback.print_exc()
        return None



#!/usr/bin/python
# -*- coding: utf-8 -*-
from time import sleep

from homevee.DeviceAPI import philips_hue
from homevee.db_utils import get_database_con


def load_hue_values(db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM PHILIPS_HUE_LIGHTS")
        for item in cur.fetchall():
            counter = 3
            for i in range(0, counter):
                if(philips_hue.get_light_info(item['ID'], db)):
                    break



def init_thread():
    while True:
        db = get_database_con()
        load_hue_values(db)
        db.close()
        sleep(5*60)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import random

import time
import urllib2
import codecs
import json
import random
import urllib
import urllib2

CONTEXT_VALIDITY = 60

SMART_API_PATH = "https://api.homevee.de/api.php"

def contains_pattern(patterns, text):
    for pattern in patterns:
        #print pattern
        if is_in_order(pattern, text):
            return True
    return False

def is_in_order(words, text):
    indices = []

    word_parts = text.split(" ")

    for word in words:
        found = False
        if isinstance(word, list):
            for word_item in word:
                if found:
                    break
                #print word_item

                if(word_item in word_parts):
                    if(len(indices) > 0):
                        max_index = indices[len(indices)-1]
                        indices.append(word_parts.index(word_item, max_index+1))
                    else:
                        indices.append(word_parts.index(word_item))
                    #print word_item+" found"
                    found = True

            if not found:
                return False
        else:
            #print word

            if (word in word_parts):
                if(len(indices) > 0):
                    max_index = indices[len(indices)-1]
                    indices.append(word_parts.index(word, max_index+1))
                else:
                    indices.append(word_parts.index(word))
                #print word+" found"
            else:
                return False

    last_value = -1
    for index in indices:
        if index is -1 or index <= last_value:
            return False
        last_value = index
    return True

def is_in_order_backup(words, text):
    indices = []

    for word in words:
        found = False
        if isinstance(word, list):
            for word_item in word:
                if found:
                    break
                #print word_item
                position = text.find(word_item)
                if position is not -1:
                    indices.append(position)
                    found = True
            if not found:
                return False
        else:
            #print word
            position = text.find(word)
            if position is not -1:
                indices.append(position)
            else:
                return False

    last_value = -1
    for index in indices:
        if index is -1 or index <= last_value:
            return False
        last_value = index
    return True

def get_no_permission_text():
    return random.choice(["Du hast nicht die nötigen Berechtigungen für diese Aktion.",
		"Dazu fehlen dir die Berechtigungen.",
		"Das darfst du nicht."])

def classify_voice_command(request):
    url = "https://assistant.homevee.de:444/"
    data = urllib2.urlopen(url + urllib.quote(codecs.encode(request, 'utf-8'))).read()
    return json.loads(data)

def generate_string(answer_data):
    if not isinstance(answer_data, list):
        return str(answer_data)

    chosen_answer = random.choice(answer_data)

    output = ""

    if isinstance(chosen_answer, list):
        for part in chosen_answer:
            output = output + generate_string(part)
    else:
        return str(chosen_answer)
    return str(output)

def set_context(username, context_key, context_data, db):

    print context_data

    context_data = json.dumps(context_data)

    with db:
        cur = db.cursor()

        cur.execute("DELETE FROM 'VOICE_COMMAND_USER' WHERE USERNAME == :user",
                    {'user': username})

        cur.execute("INSERT INTO VOICE_COMMAND_USER (USERNAME, CONTEXT_TIME, CONTEXT_KEY, CONTEXT_DATA) VALUES (:username, :time, :key, :data)",
                    {'username': username, 'time': int(time.time()), 'key': context_key, 'data': context_data})

def get_context(username, db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM 'VOICE_COMMAND_USER' WHERE USERNAME == :user",
                    {'user': username})

        data = cur.fetchone()

        if data:
            context_data = data

            if(int(time.time()) - int(context_data['CONTEXT_TIME'])) <= CONTEXT_VALIDITY:
                cur.execute("DELETE FROM 'VOICE_COMMAND_USER' WHERE USERNAME == :user",
                            {'user': username})

                return context_data
            return None
        else:
            return None

def find_room(text, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM ROOMS")

        for room in cur.fetchall():
            position = text.find(room['NAME'].lower());
            if position is not -1:
                return room
    return None

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False

def get_okay():
    try:
        url = SMART_API_PATH + "/?action=okay"
        print url

        response = urllib2.urlopen(url)
        data = response.read()

        if data is not None:
            return data
    except urllib2.HTTPError, e:
        print e
        return ['Sehr gern', 'Ok', 'Alles klar']

def replace_voice_commands(text, username, db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM VOICE_COMMAND_REPLACE WHERE USERNAME = :user",
                    {'user': username})

        for item in cur.fetchall():
            text = text.replace(item['TEXT'], item['REPLACE_WITH'])

        return text

def user_is_at_home(username, db):
    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM USERDATA WHERE USERNAME = :username", {'username': username})

        user = cur.fetchone()

        return user['AT_HOME']


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib2

import re

from homevee.VoiceAssistant.helper import generate_string, is_number
from homevee.VoiceAssistant.voice_patterns import PATTERN_WIKIPEDIA

MIN_OUTPUT_LENGTH = 100

def get_pattern(db):
    return PATTERN_WIKIPEDIA

def get_label():
    return "wikipedia"

def run_command(username, text, context, db):
    return voice_definition(username, text, context, db)

def voice_definition(username, text, context, db):
    words = text.split(" ")

    definition = []

    query = None

    is_query = False

    for word in words:
        if is_query:
            if query is None:
                query = word.capitalize()
            else:
                query = query + " " + word.capitalize()

        if word == "ist" or word == "war":
            is_query = True

    url = 'https://de.wikipedia.org/w/api.php?format=json&action=query&prop=extracts&exintro=&explaintext=&titles='+query.replace(" ", "%20")
    response = urllib2.urlopen(url).read()

    print(url)

    print(response)

    data = json.loads(response)

    pages = data['query']['pages']

    page_keys = list(pages.keys())[0]

    if 'extract' in pages[page_keys]:
        definition = pages[page_keys]['extract']

    if  page_keys == -1 or len(definition) == 0:
        answers = [
            ['Ich habe ', ['zu ', 'über '], ' ', query, ' ' , ['leider ', 'bedauerlicherweise ', ''], [[['gar ', ''], 'nichts '], 'keine Ergebnisse '], ['gefunden', 'finden können'], '.']
        ]

        output = generate_string(answers)
        return {'msg_speech': output, 'msg_text': output}

    definition = definition.replace('[[', '')
    definition = definition.replace(']]', '')
    definition = definition.replace('\n', '')

    definition = re.sub("\s*[\(\[].*?[\)\]]\s*", " ", definition)
    definition = re.sub("\s+", " ", definition)

    #definition = re.sub('#\s*\(.+\)\s*#U', ' ', definition)
    #definition = re.sub('#\s*\[.+\]\s*#U', ' ', definition)

    sentences = definition.split('. ')

    output = sentences[0]

    last_char = output[-1:]

    index = 1
    if len(sentences) > 1:
        while is_number(last_char) or len(output) < MIN_OUTPUT_LENGTH:
            output = output + '. ' + sentences[index]
            index = index + 1
            last_char = output[-1:]

    output = output + "."

    return {'msg_speech': output, 'msg_text': output}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
from time import sleep

from homevee.db_utils import get_database_con
from homevee.device_types import *

def save_sensor_data(db):
    with db:
        cur = db.cursor()

        #Z-Wave Sensoren
        cur.execute("SELECT * FROM ZWAVE_SENSOREN")
        for item in cur.fetchall():
            if item['SAVE_DATA']:
                save_to_db(ZWAVE_SENSOR, item['ID'], item['VALUE'], db)
                
        #MQTT Sensoren
        cur.execute("SELECT * FROM MQTT_SENSORS")
        for item in cur.fetchall():
            if item['SAVE_DATA']:
                save_to_db(MQTT_SENSOR, item['ID'], item['LAST_VALUE'], db)

def save_to_db(type, id, value, db):
    with db:
        time = datetime.datetime.now().strftime('%Y-%m-%d %H:00')

        cur = db.cursor()

        cur.execute("INSERT INTO SENSOR_DATA (DEVICE_ID, DEVICE_TYPE, DATETIME, VALUE) \
                        VALUES (:id, :type, :time, :value)",
                    {'id': id, 'type': type, 'time': time, 'value':value})

def init_thread():
    while True:
        db = get_database_con()

        t = datetime.datetime.today()

        seconds_to_wait = (60*60) - (t.minute*60) - t.second
        sleep(seconds_to_wait)
        print("Saving sensor-data")
        save_sensor_data(db)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.DeviceAPI.zwave.get_devices import get_device_value
from homevee.device_types import *
from homevee.Helper.helper_functions import has_permission


def get_modes(username, room, type, id, db):
    if not has_permission(username, room, db):
        return {'status': 'nopermission'}

    if id == "":
        modi = []

        with db:
            cur = db.cursor()

            #Funksteckdosen laden
            cur.execute("SELECT * FROM FUNKSTECKDOSEN WHERE ROOM == :room", {'room': room})
            for item in cur.fetchall():
                mode_item = {'device': item['DEVICE'], 'mode': item['ZUSTAND'], 'icon': item['ICON'],
                    'name': item['NAME'], 'type': FUNKSTECKDOSE}
                modi.append(mode_item)

            #Z-Wave Schalter
            cur.execute("SELECT * FROM ZWAVE_SWITCHES WHERE LOCATION == :room", {'room': room})
            for item in cur.fetchall():
                mode_item = {'device': item['ID'], 'mode': get_modes(username, room, "Z-Wave Schalter", item['ID'], db), 'icon': item['ICON'],
                             'name': item['NAME'], 'type': ZWAVE_SWITCH}
                modi.append(mode_item)

            cur.close()

            return {'modi': modi}
    else:
        with db:
            cur = db.cursor()

            if type == FUNKSTECKDOSE:
                cur.execute("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE == :device", {'device': id})
                return cur.fetchone()['ZUSTAND']
            elif type == ZWAVE_SWITCH:
                '''gateway_data = get_gateway("Z-Wave", db)

                port = ""

                if gateway_data['PORT'] is not None and gateway_data['PORT'] != "":
                    port = ":"+gateway_data['IP'];

                link = "http://"+gateway_data['IP']+port+"/ZAutomation/api/v1/devices/"+id

                result = "{}"

                array = json.loads(result)

                if "code" not in array or array['code'] != "200":
                    return "false"

                return str(array['data']['metrics']['level'] == "on")'''
                return get_device_value(ZWAVE_SWITCH, id, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-
import random

import datetime

import re

from homevee.VoiceAssistant.helper import generate_string
from homevee.VoiceAssistant.voice_patterns import PATTERN_DATE_WEEKDAY


def get_pattern(db):
    return PATTERN_DATE_WEEKDAY

def get_label():
    return "calculator"

def run_command(username, text, context, db):
    return get_weekday(username, text, context, db)

def get_weekday(username, text, context, db):
    wochentage = ['Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag', 'Sonntag']

    month_replacements = [
        ['. januar ', '.01.'],
        ['. februar ', '.02.'],
        ['. märz ', '.03.'],
        ['. april ', '.04.'],
        ['. mai ', '.05.'],
        ['. juni ', '.06.'],
        ['. juli ', '.07.'],
        ['. august ', '.08.'],
        ['. september ', '.09.'],
        ['. oktober ', '.10.'],
        ['. november ', '.11.'],
        ['. dezember ', '.12.'],
    ]

    for replacement in month_replacements:
        text = text.replace(replacement[0], replacement[1])

    date = re.search('\d{2}\.\d{2}\.\d{4}', text)
    if(date is None):
        answer_data = [
            ['Du hast kein Datum ', ['gesagt', 'genannt'],'.'],
            ['Du musst', [' mir ', ' '], ['zuerst', 'erst', 'erst einmal'], ' ein Datum ', ['nennen', 'sagen'], '.']
        ]

        output = generate_string(answer_data)
    else:
        date_formatted = str(datetime.datetime.strptime(date.group(), '%d.%m.%Y').date())

        print(date_formatted)

        date_data = date_formatted.split('-')
        time = datetime.datetime(int(date_data[0]), int(date_data[1]), int(date_data[2]))
        wochentag = wochentage[time.weekday()]

        print(time.weekday())

        answer_data = [
            [[['Der ', date_formatted], 'Das'], ' ist ein ', wochentag, [', '+username, ''], '.']
        ]

        output = generate_string(answer_data)

    return {'msg_speech': output, 'msg_text': output}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import os
import socket
import sys
import time
import traceback
from thread import start_new_thread
import cronjobs
from DeviceAPI import mqtt_api
#from Functions import ar_control, people_classifier
#from Helper import compression
from Helper.helper_functions import save_request_to_db, parse_http_headers, send_to_client, \
    update_ip_thread, update_cert_thread, check_cert
from homevee.API import process_data
import VoiceAssistant
from homevee.cloud_communication import cloud_connection_loop
from homevee.constants import END_OF_MESSAGE
from homevee.db_utils import get_database_con
from homevee.smart_speaker_communication import smart_speaker_loop

class Homevee():
    #add flags
    def __init__(self, cloud_connection=True, websocket_server=True, http_server=True):
        self.cloud_connection = cloud_connection
        self.websocket_server = websocket_server
        self.http_server = http_server

    def start_server(self):
        HOST = ''  # Symbolic name meaning all available interfaces
        PORT = 7777  # Arbitrary non-privileged port

        # Zeitzone anpassen
        os.environ['TZ'] = 'Europe/Berlin'

        '''while True:
            data = raw_input("Data: ")
            key = raw_input("Key: ")

            aes = AESCipher.AESCipher(key)
            new_aes = AESCipher.AESCipher(key)
            print("Data: "+data)
            cipher = aes.encrypt(data)
            print("Cipher: "+cipher)
            plain = new_aes.decrypt(cipher)
            print("Plain: "+plain)'''

        # aes = AESCipher.AESCipher('3s6v9y$B&E)H@McQfTjWmZq4t7w!z%C*')
        # print("Plain: "+aes.decrypt('qYqXviw/9yowSeMeJqkRhC6QUtQdPjFhyUdlzb0Cyz4xddOQbLMpNUjs8hMbk8al'))

        # cronjobs.save_energy_data.init_thread()

        test_speech = False
        if test_speech:
            while (True):
                input = raw_input("Gib einen Sprachbefehl ein: ")
                print(VoiceAssistant.voice_command_cloud("sascha", input, None, None, get_database_con()))

        # Smart Speaker Communication
        start_new_thread(smart_speaker_loop, ())

        # Connect to MQTT-Broker
        start_new_thread(mqtt_api.init_client, ())

        # Start Websocket-Server
        if self.websocket_server:
            pass
            # start_new_thread(start_websocket_server, ())

        # Start HTTP-Server
        if self.http_server:
            pass
            # start_new_thread(webinterface.start_http_server, ())

        # Start cronjobs
        cronjobs.start_cronjobs()

        # Communicate with cloud
        if self.cloud_connection:
            start_new_thread(cloud_connection_loop, ())

        # Lokale IP und Zertifikat regelmäßig aktualisieren
        start_new_thread(update_ip_thread, ())
        start_new_thread(update_cert_thread, ())

        # Reset Image-Classifier flags
        # db = get_database_con()
        # set_server_data(ar_control.IS_TRAINING_TAG, "false", db)
        # set_server_data(people_classifier.IS_TRAINING_TAG, "false", db)

        # check if certs exist
        check_cert(db=get_database_con())

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        print('Socket created')

        # Bind socket to local host and port
        try:
            s.bind((HOST, PORT))
        except socket.error as msg:
            traceback.print_exc()
            print('Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1])
            sys.exit()

        print('Socket bind complete')

        # Start listening on socket
        s.listen(10)
        print('Socket now listening')

        while True:
            try:
                self.listen_for_requests(s)
            except KeyboardInterrupt:
                break
            except:
                traceback.print_exc()

        s.close()

    def websocket_client(self, client, server):
        server.set_fn_message_received(self.handle_websocket_msg)

    def start_websocket_server(self):
        server = websocket_server.WebsocketServer(7889, host='')
        server.set_fn_new_client(self.websocket_client)
        server.run_forever()

    def handle_websocket_msg(self, client, server, data):
        # infinite loop so that function do not terminate and thread do not end.
        db = get_database_con()

        sent_data = None

        while True:
            try:
                # Receiving from client
                method_start_time = time.time()

                data = data.decode("utf-8")

                if data == "":
                    db.close()
                    return

                print("Received: " + str(data))

                is_http = False
                is_resend = False

                error = False

                # Ist HTTP-Aufruf?
                try:
                    data = parse_http_headers(data)
                    is_http = True
                except AttributeError:
                    try:
                        data = json.loads(data)
                        data = json.loads(data['msg'])
                        print("Request is not in HTTP-Format")
                    except:
                        print("Data could not be parsed")
                        server.send_message(client, json.dumps({'status': 'error'}))
                        error = True

                '''if 'resend' in data and data['resend'] is True:
                    # Daten erneut senden
                    print("Resend!")
                    is_resend = True
                    print(str(len(sent_data)) + " | " + sent_data)

                    send_to_client(sent_data[data['resendstart']:], conn, is_http)'''

                if not error and not is_resend:
                    if not is_resend:
                        reply = process_data(data, db)

                        reply['computing_time'] = time.time() - method_start_time

                    if (reply is not None):
                        server.send_message(client, json.dumps(reply))

                        save_request_to_db(data, reply, db)
                    else:
                        print("Error in reply")
                        server.send_message(client, json.dumps({'status': 'error'}))

                    if not data:
                        break
            # except socket.error:
            #    break
            except Exception as e:
                traceback.print_exc()
                print("Fehler: " + str(e))
                server.send_message(client, json.dumps({'status': 'error'}))

            # Stop endless loop
            break

        # came out of loop
        db.commit()
        db.close()
        print("Verbindung beendet")

    # Function for handling connections. This will be used to create threads
    def clientthread(self, conn):
        # infinite loop so that function do not terminate and thread do not end.
        db = get_database_con()

        is_http = False

        sent_data = None

        while True:
            try:
                # Receiving from client
                method_start_time = time.time()

                data = ""

                while (True):
                    new_data = conn.recv(8192)

                    # print(new_data)

                    # data = data + compression.decompress_string(new_data)

                    data = data + new_data

                    if (data.endswith(END_OF_MESSAGE)):
                        break

                if data == "":
                    db.close()
                    conn.close()
                    return

                data = data[:len(END_OF_MESSAGE) * -1]

                data = data.decode("utf-8")

                print("Received: " + str(data))

                is_http = False
                is_resend = False

                error = False

                # Ist HTTP-Aufruf?
                try:
                    data = parse_http_headers(data)
                    is_http = True
                except AttributeError:
                    try:
                        data = json.loads(data)
                        data = json.loads(data['msg'])
                        print("Request is not in HTTP-Format")
                    except:
                        print("Data could not be parsed")
                        send_to_client(json.dumps({'status': 'error'}), conn, is_http)
                        error = True

                if 'resend' in data and data['resend'] is True:
                    # Daten erneut senden
                    print("Resend!")
                    is_resend = True
                    print(str(len(sent_data)) + " | " + sent_data)

                    send_to_client(sent_data[data['resendstart']:], conn, is_http)

                if not error and not is_resend:
                    if not is_resend:
                        message = process_data(data, db)

                        # print "compressing: "+message

                        # start_time = time.time()
                        # compressed_message = compression.compress_string(message)
                        # end_time = time.time()

                        # print "uncompressed: "+str(len(message))
                        # print "compressed: "+str(len(compressed_message))+", time: "+str(end_time-start_time)

                        # compressed_message = compressed_message.decode('iso-8859-1').encode('utf8')

                        msg = json.dumps({'msg': message, 'computing_time': (time.time() - method_start_time) * 1000})

                    if (msg is not None):
                        send_to_client(msg, conn, is_http)

                        sent_data = msg

                        save_request_to_db(data, msg, db)
                    else:
                        print("Error in reply")
                        send_to_client(json.dumps({'status': 'error'}), conn, is_http)

                    if not data:
                        break
            # except socket.error:
            #    break
            except Exception as e:
                traceback.print_exc()
                print("Fehler: " + str(e))
                send_to_client(json.dumps({'status': 'error'}), conn, is_http)

            # Stop endless loop
            break

        # came out of loop
        conn.close()
        db.commit()
        db.close()
        print("Verbindung beendet")

    def listen_for_requests(self, s):
        # s = ssl.wrap_socket(s, keyfile=constants.LOCAL_SSL_PRIVKEY, certfile=constants.LOCAL_SSL_CERT, server_side=True)

        # wait to accept a connection - blocking call
        conn, addr = s.accept()
        print('Connected with ' + addr[0] + ':' + str(addr[1]))

        # start new thread takes 1st argument as a function name to be run, second is the tuple of arguments to the function.
        start_new_thread(self.clientthread, (conn,))


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.VoiceAssistant.voice_patterns import PATTERN_PLACES


def get_pattern(db):
    return PATTERN_PLACES

def get_label():
    return "location"

def run_command(username, text, context, db):
    return get_places(username, text, context, db)

def get_places(username, text, context, db):
    return {'msg_speech': 'Places', 'msg_text': 'Places'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import codecs
import traceback
import urllib
import urllib2

import homevee.VoiceAssistant
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.helper import generate_string

GREETINGS = ['hallo', 'hey', 'na', 'hi']

THANKS = ['danke', 'dank', 'dankeschön']

def get_pattern(db):
    return []

def get_label():
    return "conversation"

def run_command(username, text, context, db):
    return conversation(username, text, context, db)

def conversation(username, text, context, db):
    url = helper.SMART_API_PATH + "/?action=plaintext&text="+urllib.quote(codecs.encode(text, 'utf-8'))
    #print url
    output = urllib2.urlopen(url).read()

    #print output

    if output is None or output == '':
        data = [
            [[['Es ', 'tut '], 'Tut '], 'mir ', ['echt ', 'sehr ', ''], ['Leid. ', 'Leid, '+username+'. '],
            ['Da ', 'Dabei ', 'Damit '], 'kann ', 'ich ', 'dir ', ['leider ', ''], ['noch ', ''], 'nicht ', ['helfen', 'weiterhelfen'], '.']
        ]
        output = generate_string(data)


    result = {'msg_speech': output, 'msg_text': output}

    return result

def contains(text, array):
    words = text.split(' ')

    for item in array:
        if item in words:
            return True

    return False

def get_label():
    return "conversation"


import time
from platform   import system as system_name  # Returns the system/OS name
from subprocess import call   as system_call  # Execute a shell command
from homevee import db_utils


def init_thread():
    while(True):
        ping_users(db_utils.get_database_con())
        time.sleep(15)

def ping_users(db):
    with db:
        cur = db.cursor()

        cur.execute("SELECT * FROM USERDATA")

        items = cur.fetchall()

        for item in items:
            ip = item['IP']

            print "Pinging user: "+item['USERNAME']

            last_val = item['AT_HOME']

            at_home = ping(ip)

            #if last_val != at_home:
            #   trigger automation

            if at_home:
                print item['USERNAME']+ ": at home"
            else:
                print item['USERNAME']+ ": not at home"

            cur.execute("UPDATE USERDATA SET AT_HOME = :at_home WHERE USERNAME = :user",
                        {'at_home': at_home, 'user': item['USERNAME']})

def ping(host):
    """
    Returns True if host (str) responds to a ping request.
    Remember that a host may not respond to a ping (ICMP) request even if the host name is valid.
    """

    # Ping command count option as function of OS
    param = '-n' if system_name().lower()=='windows' else '-c'

    # Building the command. Ex: "ping -c 1 google.com"
    command = ['ping', param, '1', host]

    # Pinging
    return system_call(command) == 0


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import urllib

from homevee.Manager.gateway import get_gateway
from homevee.gateway_keys import RADEMACHER_HOMEPILOT

import heating_control
import blinds_control

def get_devices(db):
    data = get_gateway(RADEMACHER_HOMEPILOT, db)
    ip = data['IP']

    devices = []

    url = "http://"+ip+"/deviceajax.do?devices=1"

    response = urllib.urlopen(url).read()

    data = json.loads(response)

    for device in data['devices']:
        item = {'title': device['name'], 'id': device['did'], 'info': device['description']}

        devices.append(item)

    return {'devices': devices}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import re

from homevee.Functions.nutrition_data import get_nutrition_data, add_edit_user_day_nutrition_item, get_user_nutrition_overview, \
    get_user_fitness_profile
from homevee.VoiceAssistant.context_keys import ADD_NUTRITION
from homevee.VoiceAssistant.helper import set_context, generate_string
from homevee.VoiceAssistant.voice_patterns import PATTERN_NUTRITION_INFO

MAX_PORTION_DEVIATION = 10

def get_pattern(db):
    return PATTERN_NUTRITION_INFO

def get_label():
    return "nutritiondata"

def run_command(username, text, context, db):
    return voice_get_nutrition_info(username, text, context, db)

def voice_add_nutrition_item(username, text, context, db):
    answer = "add_nutrition_item"

    text = re.sub(r'([\d]+)g', '\1 Gramm', text)

    print text

    words = text.split(" ")

    number_words = ['ein', 'einer', 'eine', 'einem', 'einen']

    amount_words = ['gramm', 'milliliter', 'kilo', 'kilogramm', 'liter', 'stück', 'scheiben', 'scheibe', 'brötchen']

    eat_words = ['esse', 'gegessen', 'essen', 'ess', 'trink', 'trinke', 'trinken', 'getrunken']

    time_words = ['jetzt', 'gerade', 'eben']

    daytime_words = ['morgens', 'morgen', 'mittag', 'mittags', 'abend', 'abends', 'snack', 'snacks', 'zwischendurch']

    is_name = False

    item = None

    amount = None
    amount_type = None

    daytime = None

    current_hour = datetime.datetime.now().hour

    for word in words:
        if(word in time_words):
            daytime = 'snacks'
            if current_hour >= 5 and current_hour < 10:
                daytime = 'morning'
            elif current_hour >= 11 and current_hour < 13:
                daytime = 'noon'
            elif current_hour >= 17 and current_hour < 19:
                daytime = 'evening'

        if(word in daytime_words and daytime is None):
            if word in ['morgens', 'morgen']:
                daytime = 'morning'
            elif word in ['mittag', 'mittags']:
                daytime = 'noon'
            elif word in ['abends', 'abend']:
                daytime = 'evening'
            elif word in ['snack', 'snacks', 'zwischendurch']:
                daytime = 'snacks'

        if is_name and word in eat_words:
            is_name = False

        if is_name:
            if item is None:
                item = word
            else:
                item = item + " " +word

        if word in number_words:
            is_name = True
            amount = 1

        if word in amount_words:
            amount = words[words.index(word)-1]
            amount_type = word
            is_name = True

    print str(item)+" => "+str(amount)+" => "+str(amount_type)+" => "+str(daytime)

    nutrition_data = None

    if(context is not None):

        if('query' in context):
            if(context['query'] == 'amount'):
                for word in words:
                    if word in number_words:
                        amount = 1
                        break

                if amount is None:
                    numbers_in_text = [int(s) for s in text.split() if s.isdigit()]

                    if (len(numbers_in_text) != 0):
                        amount = numbers_in_text[0]
            elif(context['query'] == 'item'):
                item = text
            elif(context['query'] == 'daytime'):
                for word in words:
                    if (word in daytime_words and daytime is None):
                        if word in ['morgens', 'morgen']:
                            daytime = 'morning'
                        elif word in ['mittag', 'mittags']:
                            daytime = 'noon'
                        elif word in ['abends', 'abend']:
                            daytime = 'evening'
                        elif word in ['snack', 'snacks', 'zwischendurch']:
                            daytime = 'snacks'

        if daytime is None and 'daytime' in context and context['daytime'] is not None:
            daytime = context['daytime']

        if amount is None and 'amount' in context and context['amount'] is not None:
            amount = context['amount']

        if amount_type is None and 'amount_type' in context and context['amount_type'] is not None:
            amount_type = context['amount_type']

        if nutrition_data is None and 'nutrition_data' in context and context['nutrition_data'] is not None:
            nutrition_data = context['nutrition_data']
            item = nutrition_data['name']

    #print "Amount: "+str(amount)

    if(item is not None):
        if(nutrition_data is None):
            nutrition_data = get_nutrition_data(username, item, db)

        if(nutrition_data is None or nutrition_data is False):
            answer = "Zum Suchbegriff "+item+" wurden keine Einträge gefunden."
            return {'msg_speech': answer, 'msg_text': answer}

        date = datetime.datetime.today().strftime("%d.%m.%Y")

        if(amount is None):
            answer = "Wie viel " + nutrition_data['portionunit'] + " hast du von " + nutrition_data[
                'name'] + " verzehrt?"
            context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type,
                       'daytime': daytime, 'query': 'amount'}
            set_context(username, ADD_NUTRITION, context, db)
        elif(daytime is None):
            answer = "Wann hast du " + nutrition_data['name'] + " verzehrt? Morgens, Mittags, Abends oder zwischendurch?"
            context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type,
                       'daytime': daytime, 'query': 'daytime'}
            set_context(username, ADD_NUTRITION, context, db)
        else:
            #print "amount: "+str(amount)
            portion_deviation = float(amount) / float(nutrition_data['portionsize'])

            #print portion_deviation

            if(portion_deviation >= MAX_PORTION_DEVIATION or portion_deviation <= 1.0/float(MAX_PORTION_DEVIATION)):
                #invalid portion size

                answer = "Wie viel "+nutrition_data['portionunit']+" hast du von "+nutrition_data['name']+" verzehrt?"
                context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type, 'daytime': daytime, 'query': 'amount'}
                set_context(username, ADD_NUTRITION, context, db)
            else:
                #anhand der aktuellen uhrzeit die daytime errechnen

                answer = "Ok, " + str(amount) + " " + nutrition_data['portionunit'] + " "+ nutrition_data['name'] +" wurde deinem Ernährungstagebuch hinzugefügt."

                add_edit_user_day_nutrition_item(username, None, date, daytime, nutrition_data['name'], amount,
                                             nutrition_data['portionsize'], nutrition_data['portionunit'],
                                             nutrition_data['calories'], nutrition_data['fat'], nutrition_data['saturated'],
                                             nutrition_data['unsaturated'], nutrition_data['carbs'], nutrition_data['sugar'],
                                             nutrition_data['protein'], db)

                calories_left = get_user_nutrition_overview(username, db)['data']['nutrition_day_data']['calories_left']

                if (calories_left >= 0):
                    answer = answer + ' Du hast heute noch ' + str(calories_left) + ' Kalorien übrig.'
                else:
                    answer = answer + ' Du bist heute ' + str(-1*calories_left) + ' Kalorien über deinem Ziel.'
    else:
        answer = "Was möchtest du deinem Ernährungstagebuch hinzufügen?"
        context = {'amount': amount, 'amount_type': amount_type, 'daytime': daytime, 'query': 'item'}
        set_context(username, ADD_NUTRITION, context, db)

    #determine amount

    #determine product

    return {'msg_speech': answer, 'msg_text': answer}

def voice_get_nutrition_info(username, text, context, db):
    words = text.split(" ")

    number_words = ['ein', 'einer', 'eine', 'einem', 'einen']

    if 'in' in words:
        in_index = words.index('in')
        del words[in_index]

        index = None

        for number_word in number_words:
            if number_word in words:
                index = words.index(number_word)

        if index is not None:
            del words[index]

    nutrition_types = ['fett', 'eiweiß', 'protein', 'kohlenhydrate', 'kalorien', 'zucker', 'proteine']

    nutrition_value = None

    for word in words:
        if word in nutrition_types:
            nutrition_value = word
            break

    if nutrition_value is None:
        return {'msg_speech': 'Es gab einen Fehler.',
                'msg_text': 'Es gab einen Fehler.'}

    item = None
    trigger = ['ist', 'sind', 'haben', 'hat']
    add_word = False
    add_word_index = None
    amount_type = None
    amount = None
    for i in range(0, len(words)):
        word = words[i]

        if add_word_index is not None and i == add_word_index:
            add_word = True

        if add_word:
            if word in number_words:
                continue

            if item is None:
                item = word
            else:
                item = item + " " + word
            continue

        if word in trigger:
            if 'gramm' in words:
                index = words.index('gramm')
                amount_type = 'gramm'
            elif 'kili' in words:
                index = words.index('kilo')
                amount_type = 'kilo'
            elif 'milliliter' in words:
                index = words.index('milliliter')
                amount_type = 'milliliter'
            elif 'liter' in words:
                index = words.index('liter')
                amount_type = 'liter'
            else:
                add_word = True
                continue

            amount = words[index - 1]
            add_word_index = index + 1

    answer = ""

    # Daten abfragen
    nutrition_data = get_nutrition_data(username, item, db)

    #print nutrition_data

    if nutrition_data is not None:
        if nutrition_data is not False:
            portionsize = nutrition_data['portionsize']

            if portionsize == 1:
                portionsize_string = "Ein"
            else:
                portionsize_string = str(portionsize)

            portion_string = portionsize_string + " " + nutrition_data['portionunit']

            name = nutrition_data['name']

            if (nutrition_value == "fett"):
                value = nutrition_data['fat']
                unit = "Gramm Fett"
            elif (nutrition_value in ['eiweiß', 'protein', 'proteine']):
                value = nutrition_data['protein']
                unit = "Gramm Eiweiß"
            elif (nutrition_value == "kohlenhydrate"):
                value = nutrition_data['carbs']
                unit = "Gramm Kohlenhydrate"
            elif (nutrition_value == "zucker"):
                value = nutrition_data['sugar']
                unit = "Gramm Zucker"
            elif (nutrition_value == "kalorien"):
                value = nutrition_data['calories']
                unit = "Kalorien"

            have_verb = "hat"

            if (portionsize > 1):
                have_verb = "haben"

            answer = portion_string + " " + name + " " + have_verb + " " + str(value) + " " + unit + "."

            context_data = {'nutrition_type': nutrition_value}
            # set_context(username, 'nutrition', context_data, db)
        else:
            answer = "Es wurden keine Lebensmittel zum Suchbegriff "+item+" gefunden."

    else:
        #fehlermeldung ausgeben

        answer = "Die Nährwertdaten konnten nicht abgefragt werden."

    amount_text = ''
    if amount_type is not None and amount is not None:
        amount_text = ' => ' + amount + ' ' + amount_type

    return {'msg_speech': answer, 'msg_text': answer}

def voice_query_nutrition_diary(username, text, context, db):
    words = text.split(" ")

    queried_value = None

    for word in words:
        if word == 'fett':
            queried_value = 'FAT'
        elif word == 'kalorien':
            queried_value = 'CALORIES'
        elif word == 'kohlenhydrate':
            queried_value = 'CARBS'
        elif word == 'zucker':
            queried_value = 'SUGAR'
        elif word in ['protein', 'proteine', 'eiweiß']:
            queried_value = 'PROTEIN'
        else:
            continue

        break

    user_profile = get_user_fitness_profile(username, db)

    print user_profile

    if (user_profile is False):
        answer = "Du hast noch kein Profil für den Ernährungsmanager erstellt. Das kannst du in der App im Menüpunkt Ernährungsmanager tun."
    elif queried_value is None:
        #Kein Wert gefunden
        answer = "Welchen Nährwert möchtest du abfragen?"

        answer_data = [
            ['Welcher Nährwert soll abgefragt werden.'],
            ['Welchen Nährwert ', ['magst', 'möchtest', 'willst'],' du abfragen.'],
        ]
    else:
        with db:
            cur = db.cursor()
            cur.execute("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                        {'user': username, 'date': datetime.datetime.today().strftime("%d.%m.%Y")})

            eaten_queried_value_today = 0

            for item in cur.fetchall():
                amount = (float(item['EATEN_PORTION_SIZE']) / float(item['PORTIONSIZE']))

                eaten_queried_value_today += item[queried_value] * amount

            unit = None

            if queried_value == 'FAT':
                value_left = user_profile['fatgoal']-eaten_queried_value_today
                unit = "Gramm Fett"
            elif queried_value == 'CALORIES':
                value_left = int(user_profile['caloriesgoal']-eaten_queried_value_today)
                eaten_queried_value_today = int(eaten_queried_value_today)
                unit = 'Kalorien'
            elif queried_value == 'CARBS':
                value_left = user_profile['carbsgoal']-eaten_queried_value_today
                unit = 'Gramm Kohlenhydrate'
            elif queried_value == 'SUGAR':
                value_left = user_profile['sugargoal']-eaten_queried_value_today
                unit = 'Gramm Zucker'
            elif queried_value == 'PROTEIN':
                value_left = user_profile['proteingoal']-eaten_queried_value_today
                unit = 'Gramm Protein'

            answer_data = [
                ['Du hast heute ', ['schon', 'bereits'], ' ', str(eaten_queried_value_today), " ", unit, ' ',
                 ['gegessen', 'verzehrt', 'aufgenommen', 'zu dir genommen'], ' und ', ['noch', 'weitere'], ' ',
                 str(value_left), ' ', unit, ' ', ['übrig', 'frei', 'offen'], '.']
            ]

            #answer = "Du hast heute schon "+str(eaten_queried_value_today)+" "+unit+" zu dir genommen und noch "+str(value_left)+" "+unit+" übrig."

    answer = generate_string(answer_data)

    return {'msg_speech': answer, 'msg_text': answer}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import re
import traceback

from homevee.Functions.calendar import get_calendar_day_items
from homevee.VoiceAssistant.helper import *
from homevee.VoiceAssistant.voice_patterns import PATTERN_GET_CALENDAR, PATTERN_ADD_CALENDAR


def get_pattern(db):
    return PATTERN_ADD_CALENDAR

def get_label():
    return "getcalendar"

def run_command(username, text, context, db):
    return add_calendar(username, text, context, db)

def add_calendar(username, text, context, db):
    return {'msg_speech': 'Add Calendar', 'msg_text': 'Add Calendar'}

def find_date(text):
    now = datetime.datetime.now()

    try:
        month_replacements = [
            ['. januar ', '.01.'],
            ['. februar ', '.02.'],
            ['. märz ', '.03.'],
            ['. april ', '.04.'],
            ['. mai ', '.05.'],
            ['. juni ', '.06.'],
            ['. juli ', '.07.'],
            ['. august ', '.08.'],
            ['. september ', '.09.'],
            ['. oktober ', '.10.'],
            ['. november ', '.11.'],
            ['. dezember ', '.12.'],

            ['.januar ', '.01.'],
            ['.februar ', '.02.'],
            ['.märz ', '.03.'],
            ['.april ', '.04.'],
            ['.mai ', '.05.'],
            ['.juni ', '.06.'],
            ['.juli ', '.07.'],
            ['.august ', '.08.'],
            ['.september ', '.09.'],
            ['.oktober ', '.10.'],
            ['.november ', '.11.'],
            ['.dezember ', '.12.'],
        ]

        for replacement in month_replacements:
            text = text.replace(replacement[0], replacement[1])

        date = re.search('\d{2}\.\d{2}\.\d{4}', text)
        date_formatted = datetime.datetime.strptime(date.group(), '%d.%m.%Y').date()

        return 'am ' + date_formatted.strftime("%d.%m.%Y"), date_formatted.strftime("%Y-%m-%d")
    except Exception, e:
        print("No date found")

        try:
            month_replacements = [
                ['. januar ', '.01. '],
                ['. februar ', '.02. '],
                ['. märz ', '.03. '],
                ['. april ', '.04. '],
                ['. mai ', '.05. '],
                ['. juni ', '.06. '],
                ['. juli ', '.07. '],
                ['. august ', '.08. '],
                ['. september ', '.09. '],
                ['. oktober ', '.10. '],
                ['. november ', '.11. '],
                ['. dezember ', '.12. '],

                ['.januar ', '.01. '],
                ['.februar ', '.02. '],
                ['.märz ', '.03. '],
                ['.april ', '.04. '],
                ['.mai ', '.05. '],
                ['.juni ', '.06. '],
                ['.juli ', '.07. '],
                ['.august ', '.08. '],
                ['.september ', '.09. '],
                ['.oktober ', '.10. '],
                ['.november ', '.11. '],
                ['.dezember ', '.12. '],
            ]

            for replacement in month_replacements:
                text = text.replace(replacement[0], replacement[1])

            date = re.search('\d{2}\.\d{2}\.', text)
            date_formatted = datetime.datetime.strptime(date.group()+str(datetime.datetime.now().year), '%d.%m.%Y').date()

            return 'am ' + date_formatted.strftime("%d.%m.%Y"), date_formatted.strftime("%Y-%m-%d")
        except Exception, e:
            traceback.print_exc()
            print("Date without year also not found")

    words = text.split()

    if "heute" in words:
        return 'heute', now.strftime("%Y-%m-%d")
    if "morgen" in words:
        return 'morgen', (now + datetime.timedelta(days=1)).strftime("%Y-%m-%d")
    if "übermorgen" in words:
        return 'übermorgen', (now + datetime.timedelta(days=2)).strftime("%Y-%m-%d")

    #when no date was found return today
    return 'heute', now.strftime("%Y-%m-%d")


#!/usr/bin/python
# -*- coding: utf-8 -*-
import SimpleHTTPServer
import SocketServer

import os

PORT = 8000

def start_http_server():
    Handler = SimpleHTTPServer.SimpleHTTPRequestHandler

    httpd = SocketServer.TCPServer(("", PORT), Handler)

    os.chdir(os.path.dirname(os.path.realpath(__file__)))

    print "HTTP-Server running on port: "+str(8000)

    httpd.serve_forever()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import re

from homevee.Functions.nutrition_data import get_nutrition_data, add_edit_user_day_nutrition_item, get_user_nutrition_overview, \
    get_user_fitness_profile
from homevee.VoiceAssistant.context_keys import ADD_NUTRITION
from homevee.VoiceAssistant.helper import set_context, generate_string
from homevee.VoiceAssistant.voice_patterns import *

MAX_PORTION_DEVIATION = 10

def get_pattern(db):
    return PATTERN_ADD_NUTRITION_ITEM

def get_label():
    return "addnutrition"

def run_command(username, text, context, db):
    return voice_add_nutrition_item(username, text, context, db)

def voice_add_nutrition_item(username, text, context, db):
    answer = "add_nutrition_item"

    text = re.sub(r'([\d]+)g', '\1 Gramm', text)

    print text

    words = text.split(" ")

    number_words = ['ein', 'einer', 'eine', 'einem', 'einen']

    amount_words = ['gramm', 'milliliter', 'kilo', 'kilogramm', 'liter', 'stück', 'scheiben', 'scheibe', 'brötchen']

    eat_words = ['esse', 'gegessen', 'essen', 'ess', 'trink', 'trinke', 'trinken', 'getrunken']

    time_words = ['jetzt', 'gerade', 'eben']

    daytime_words = ['morgens', 'morgen', 'mittag', 'mittags', 'abend', 'abends', 'snack', 'snacks', 'zwischendurch']

    is_name = False

    item = None

    amount = None
    amount_type = None

    daytime = None

    current_hour = datetime.datetime.now().hour

    for word in words:
        if(word in time_words):
            daytime = 'snacks'
            if current_hour >= 5 and current_hour < 10:
                daytime = 'morning'
            elif current_hour >= 11 and current_hour < 13:
                daytime = 'noon'
            elif current_hour >= 17 and current_hour < 19:
                daytime = 'evening'

        if(word in daytime_words and daytime is None):
            if word in ['morgens', 'morgen']:
                daytime = 'morning'
            elif word in ['mittag', 'mittags']:
                daytime = 'noon'
            elif word in ['abends', 'abend']:
                daytime = 'evening'
            elif word in ['snack', 'snacks', 'zwischendurch']:
                daytime = 'snacks'

        if is_name and word in eat_words:
            is_name = False

        if is_name:
            if item is None:
                item = word
            else:
                item = item + " " +word

        if word in number_words:
            is_name = True
            amount = 1

        if word in amount_words:
            amount = words[words.index(word)-1]
            amount_type = word
            is_name = True

    print str(item)+" => "+str(amount)+" => "+str(amount_type)+" => "+str(daytime)

    nutrition_data = None

    if(context is not None):

        if('query' in context):
            if(context['query'] == 'amount'):
                for word in words:
                    if word in number_words:
                        amount = 1
                        break

                if amount is None:
                    numbers_in_text = [int(s) for s in text.split() if s.isdigit()]

                    if (len(numbers_in_text) != 0):
                        amount = numbers_in_text[0]
            elif(context['query'] == 'item'):
                item = text
            elif(context['query'] == 'daytime'):
                for word in words:
                    if (word in daytime_words and daytime is None):
                        if word in ['morgens', 'morgen']:
                            daytime = 'morning'
                        elif word in ['mittag', 'mittags']:
                            daytime = 'noon'
                        elif word in ['abends', 'abend']:
                            daytime = 'evening'
                        elif word in ['snack', 'snacks', 'zwischendurch']:
                            daytime = 'snacks'

        if daytime is None and 'daytime' in context and context['daytime'] is not None:
            daytime = context['daytime']

        if amount is None and 'amount' in context and context['amount'] is not None:
            amount = context['amount']

        if amount_type is None and 'amount_type' in context and context['amount_type'] is not None:
            amount_type = context['amount_type']

        if nutrition_data is None and 'nutrition_data' in context and context['nutrition_data'] is not None:
            nutrition_data = context['nutrition_data']
            item = nutrition_data['name']

    #print "Amount: "+str(amount)

    if(item is not None):
        if(nutrition_data is None):
            nutrition_data = get_nutrition_data(username, item, db)

        if(nutrition_data is None or nutrition_data is False):
            answer = "Zum Suchbegriff "+item+" wurden keine Einträge gefunden."
            return {'msg_speech': answer, 'msg_text': answer}

        date = datetime.datetime.today().strftime("%d.%m.%Y")

        if(amount is None):
            answer = "Wie viel " + nutrition_data['portionunit'] + " hast du von " + nutrition_data[
                'name'] + " verzehrt?"
            context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type,
                       'daytime': daytime, 'query': 'amount'}
            set_context(username, ADD_NUTRITION, context, db)
        elif(daytime is None):
            answer = "Wann hast du " + nutrition_data['name'] + " verzehrt? Morgens, Mittags, Abends oder zwischendurch?"
            context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type,
                       'daytime': daytime, 'query': 'daytime'}
            set_context(username, ADD_NUTRITION, context, db)
        else:
            #print "amount: "+str(amount)
            portion_deviation = float(amount) / float(nutrition_data['portionsize'])

            #print portion_deviation

            if(portion_deviation >= MAX_PORTION_DEVIATION or portion_deviation <= 1.0/float(MAX_PORTION_DEVIATION)):
                #invalid portion size

                answer = "Wie viel "+nutrition_data['portionunit']+" hast du von "+nutrition_data['name']+" verzehrt?"
                context = {'nutrition_data': nutrition_data, 'amount': amount, 'amount_type': amount_type, 'daytime': daytime, 'query': 'amount'}
                set_context(username, ADD_NUTRITION, context, db)
            else:
                #anhand der aktuellen uhrzeit die daytime errechnen

                answer = "Ok, " + str(amount) + " " + nutrition_data['portionunit'] + " "+ nutrition_data['name'] +" wurde deinem Ernährungstagebuch hinzugefügt."

                add_edit_user_day_nutrition_item(username, None, date, daytime, nutrition_data['name'], amount,
                                             nutrition_data['portionsize'], nutrition_data['portionunit'],
                                             nutrition_data['calories'], nutrition_data['fat'], nutrition_data['saturated'],
                                             nutrition_data['unsaturated'], nutrition_data['carbs'], nutrition_data['sugar'],
                                             nutrition_data['protein'], db)

                calories_left = get_user_nutrition_overview(username, db)['data']['nutrition_day_data']['calories_left']

                if (calories_left >= 0):
                    answer = answer + ' Du hast heute noch ' + str(calories_left) + ' Kalorien übrig.'
                else:
                    answer = answer + ' Du bist heute ' + str(-1*calories_left) + ' Kalorien über deinem Ziel.'
    else:
        answer = "Was möchtest du deinem Ernährungstagebuch hinzufügen?"
        context = {'amount': amount, 'amount_type': amount_type, 'daytime': daytime, 'query': 'item'}
        set_context(username, ADD_NUTRITION, context, db)

    #determine amount

    #determine product

    return {'msg_speech': answer, 'msg_text': answer}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import urllib2
from time import sleep

from homevee.Manager.api_key import get_api_key
from homevee.db_utils import get_server_data, get_database_con


def refresh_weather_cache(db):
    api_key = get_api_key("Open Weather Map", db)
    location_id = get_server_data("WEATHER_LOCATION_ID", db)

    url = "http://api.openweathermap.org/data/2.5/forecast/daily?id=" + location_id + "&cnt=16&units=metric&lang=de&type=accurate&APPID=" + api_key
    response = urllib2.urlopen(url).read()

    with db:
        cur = db.cursor()

        # Wetter-Daten in Datenbank schreiben
        cur.execute("INSERT OR REPLACE INTO SERVER_DATA (KEY, VALUE) values('WEATHER_CACHE', :response);",
                    {"response": response})
        db.commit()

def init_thread():
    while True:
        db = get_database_con()
        refresh_weather_cache(db)

        t = datetime.datetime.today()

        seconds_to_wait = (15 * 60) - ((t.minute * 60) - t.second)%15*60
        #print "warten für wetter: "+str(seconds_to_wait)+" sekunden"
        sleep(seconds_to_wait)


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.DeviceAPI.rgb_control import rgb_control
from homevee.VoiceAssistant.helper import find_room
from homevee.VoiceAssistant.voice_patterns import PATTERN_RGB_CONTROL
from homevee.colors import COLORS, COLOR_NAMES
from homevee.device_types import *

def get_pattern(db):
    return PATTERN_RGB_CONTROL

def get_label():
    return "rgb"

def run_command(username, text, context, db):
    return voice_rgb_control(username, text, context, db)

def voice_rgb_control(username, text, context, db):
    room = find_room(text, db)

    if room is not None:
        room_key = room['LOCATION']
    else:
        room_key = None

    devices = find_devices(text, room_key, db)

    color = find_color(text)
    if color != False:
        color_hex = COLOR_NAMES[color]
    else:
        return "Du musst mir eine Farbe sagen."

    for device in devices:
        rgb_control(username, device['type'], device['id'], True, None, color_hex, db)

        return {'msg_speech': 'Ok.', 'msg_text': 'Ok.'}

def find_devices(text, location_key, db):
    devices = []

    with db:
        cur = db.cursor()

        #data => table_name, name_key, id_key, type_key, location_key
        data = [
            ['PHILIPS_HUE_LIGHTS','NAME','ID',PHILIPS_HUE_LIGHT,'LOCATION'],
            ['URL_RGB_LIGHT','NAME','ID',URL_RGB_LIGHT,'LOCATION']
        ]

        for item in data:
            if location_key is None:
                cur.execute("SELECT * FROM "+item[0])
            else:
                param_array = {'location': location_key}
                cur.execute("SELECT * FROM "+item[0]+" WHERE "+item[4]+" = :location", param_array)

            for device in cur.fetchall():
                position = text.find(device[item[1]].lower())
                if position is not -1:
                    device_item = {'type': item[3], 'name': device[item[1]], 'id': device[item[2]]}
                    devices.append(device_item)

    return devices

def find_color(text):
    for word in text.split():
        for color in COLORS:
            if word == color:
                return color

    return False


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import time

from homevee.DeviceAPI.heating import heating_control
from homevee.db_utils import get_database_con


def run_heating_scheme():
    db = get_database_con()

    current_datetime = datetime.datetime.now()

    with db:
        cur = db.cursor()

        #Query heating scheme for this day and time
        query = 'SELECT * FROM HEATING_SCHEME, HEATING_SCHEME_DAYS, HEATING_SCHEME_DEVICES WHERE HEATING_SCHEME.ID = HEATING_SCHEME_ID AND HEATING_SCHEME.ID = HEATING_SCHEME_DEVICES.ID AND ACTIVE = "true" AND TIME = :time AND WEEKDAY_ID = :day;'

        current_time = current_datetime.strftime("%H:%M")
        current_day = current_datetime.weekday()

        #Run command
        cur.execute(query, {'time': current_time, 'day': current_day})

        for item in cur.fetchall():
            print item

            heating_control(None, item['TYPE'], item['DEVICE_ID'], item['VALUE'], db, check_user=False)

def init_thread():
    while(True):
        run_heating_scheme()
        time.sleep(60)
    return


#!/usr/bin/python
# -*- coding: utf-8 -*-
from homevee.VoiceAssistant.voice_patterns import PATTERN_CALCULATOR

def calculator(username, text, context, db):
    return {'msg_speech':"Taschenrechner", 'msg_text':"Taschenrechner"}

def get_pattern(db):
    return PATTERN_CALCULATOR

def get_label():
    return "calculator"

def run_command(username, text, context, db):
    return calculator(username, text, context, db)


#!/usr/bin/python
# -*- coding: utf-8 -*-

from homevee.Functions.tv_data import get_tv_plan
from homevee.VoiceAssistant import helper
from homevee.VoiceAssistant.voice_patterns import PATTERN_TV

def get_pattern(db):
    return PATTERN_TV

def get_label():
    return "tvprogramm"

def run_command(username, text, context, db):
    return get_tv(username, text, context, db)

def get_tv(username, text, context, db):
    return {'msg_speech':'TV-Programm', 'msg_text':'TV-Programm'}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import re

from homevee.Functions.nutrition_data import get_nutrition_data, add_edit_user_day_nutrition_item, get_user_nutrition_overview, \
    get_user_fitness_profile
from homevee.VoiceAssistant.context_keys import ADD_NUTRITION
from homevee.VoiceAssistant.helper import set_context, generate_string
from homevee.VoiceAssistant.voice_patterns import *

MAX_PORTION_DEVIATION = 10

def get_pattern(db):
    return PATTERN_QUERY_NUTRITION_DIARY

def get_label():
    return "nutritiondiary"

def run_command(username, text, context, db):
    return voice_query_nutrition_diary(username, text, context, db)

def voice_query_nutrition_diary(username, text, context, db):
    words = text.split(" ")

    queried_value = None

    for word in words:
        if word == 'fett':
            queried_value = 'FAT'
        elif word == 'kalorien':
            queried_value = 'CALORIES'
        elif word == 'kohlenhydrate':
            queried_value = 'CARBS'
        elif word == 'zucker':
            queried_value = 'SUGAR'
        elif word in ['protein', 'proteine', 'eiweiß']:
            queried_value = 'PROTEIN'
        else:
            continue

        break

    user_profile = get_user_fitness_profile(username, db)

    print user_profile

    if (user_profile is False):
        answer = "Du hast noch kein Profil für den Ernährungsmanager erstellt. Das kannst du in der App im Menüpunkt Ernährungsmanager tun."
    elif queried_value is None:
        #Kein Wert gefunden
        answer = "Welchen Nährwert möchtest du abfragen?"

        answer_data = [
            ['Welcher Nährwert soll abgefragt werden.'],
            ['Welchen Nährwert ', ['magst', 'möchtest', 'willst'],' du abfragen.'],
        ]
    else:
        with db:
            cur = db.cursor()
            cur.execute("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                        {'user': username, 'date': datetime.datetime.today().strftime("%d.%m.%Y")})

            eaten_queried_value_today = 0

            for item in cur.fetchall():
                amount = (float(item['EATEN_PORTION_SIZE']) / float(item['PORTIONSIZE']))

                eaten_queried_value_today += item[queried_value] * amount

            unit = None

            if queried_value == 'FAT':
                value_left = user_profile['fatgoal']-eaten_queried_value_today
                unit = "Gramm Fett"
            elif queried_value == 'CALORIES':
                value_left = int(user_profile['caloriesgoal']-eaten_queried_value_today)
                eaten_queried_value_today = int(eaten_queried_value_today)
                unit = 'Kalorien'
            elif queried_value == 'CARBS':
                value_left = user_profile['carbsgoal']-eaten_queried_value_today
                unit = 'Gramm Kohlenhydrate'
            elif queried_value == 'SUGAR':
                value_left = user_profile['sugargoal']-eaten_queried_value_today
                unit = 'Gramm Zucker'
            elif queried_value == 'PROTEIN':
                value_left = user_profile['proteingoal']-eaten_queried_value_today
                unit = 'Gramm Protein'

            answer_data = [
                ['Du hast heute ', ['schon', 'bereits'], ' ', str(eaten_queried_value_today), " ", unit, ' ',
                 ['gegessen', 'verzehrt', 'aufgenommen', 'zu dir genommen'], ' und ', ['noch', 'weitere'], ' ',
                 str(value_left), ' ', unit, ' ', ['übrig', 'frei', 'offen'], '.']
            ]

            #answer = "Du hast heute schon "+str(eaten_queried_value_today)+" "+unit+" zu dir genommen und noch "+str(value_left)+" "+unit+" übrig."

    answer = generate_string(answer_data)

    return {'msg_speech': answer, 'msg_text': answer}


#!/usr/bin/python
# -*- coding: utf-8 -*-

import voice_get_sensor_data, voice_nutrition_manager, voice_places_api, voice_calculator, voice_calendar, \
    voice_conversation, voice_get_activities, voice_get_weekday, voice_jokes, voice_movie_api, voice_recipes, \
    voice_set_modes, voice_weather, voice_rgb_control, voice_tv, voice_wol, voice_route, voice_shopping_list, \
    voice_summary, voice_wikipedia, voice_tv_tipps, voice_add_calendar, voice_add_nutrition_data, voice_nutrition_diary


#!/usr/bin/python
# -*- coding: utf-8 -*-
import urllib

from Manager.gateway import get_gateway
from gateway_keys import RADEMACHER_HOMEPILOT


def set_modes(id, mode, db):
    data = get_gateway(RADEMACHER_HOMEPILOT, db)
    ip = data['IP']

    url = "http://" + ip + "/deviceajax.do?cid=9&did=10001&goto=" + str(goto) + "&command=0"

    response = urllib.urlopen(url).read()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import json
import time

import requests

import homevee.gateway_keys
from homevee.Helper.helper_functions import has_permission


def get_gateway(key, db):
    cur = db.cursor()
    cur.execute("SELECT * FROM GATEWAYS WHERE NAME == :key", {'key': key})

    data = cur.fetchone()

    cur.close()

    return data

def add_edit_gateway(username, type, user, password, change_pw, ip, port, gateway_type, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    param_array = {'name': type, 'ip': ip, 'port': port, 'key1': user, 'type': gateway_type}

    with db:
        cur = db.cursor()

        if change_pw == "true":
            param_array['key2'] = password

            cur.execute("UPDATE OR IGNORE 'GATEWAYS' SET IP = :ip, PORT = :port, KEY1 = :key1, KEY2 = :key2 TYPE = :type, WHERE NAME = :name;",
                        param_array)

            cur.execute("INSERT OR IGNORE INTO 'GATEWAYS' (NAME, IP, PORT, KEY1, KEY2, TYPE) VALUES (:name, :ip, :port, :key1, :key2, :type);",
                        param_array)
        else:
            cur.execute("UPDATE OR IGNORE 'GATEWAYS' SET IP = :ip, PORT = :port, KEY1 = :key1 WHERE NAME = :name;",
                        param_array)

            cur.execute("INSERT OR IGNORE INTO 'GATEWAYS' (NAME, IP, PORT, KEY1, TYPE) VALUES (:name, :ip, :port, :key1, :type);",
                        param_array)

    return {'result': 'ok'}

def delete_gateway(username, key, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    with db:
        cur = db.cursor()
        cur.execute("DELETE FROM GATEWAYS WHERE NAME == :key", {'key': key})

        cur.close()

        #Abfrage erfolgreich?
        if True:
            return {'status': 'ok'}
        else:
            return {'status': 'error'}

def get_gateways(username, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    with db:
        cur = db.cursor()
        cur.execute("SELECT * FROM GATEWAYS")

        gateways = []

        for gateway in cur.fetchall():
            gateways.append({'name': gateway['NAME'], 'ip': gateway['IP'], 'port': gateway['PORT'],
                             'key1': gateway['KEY1'], 'type': gateway['TYPE']})

        cur.close()

        return {'gateways': gateways, 'gatewaytypesleft': get_gateway_types(username, db)}

def get_gateway_types(username, db):
    if not has_permission(username, "admin", db):
        return {'status': 'noadmin'}

    gateway_types = [
        {'key': gateway_keys.Z_WAVE, 'type': 'user'},
        {'key': gateway_keys.FUNKSTECKDOSEN, 'type': 'url'},
        {'key': gateway_keys.MAX_CUBE, 'type': 'url'},
        {'key': gateway_keys.MQTT_BROKER, 'type': 'user'},
        {'key': gateway_keys.PHILIPS_HUE, 'type': 'apikey'},
        {'key': gateway_keys.MIYO_CUBE, 'type': 'apikey'},
        {'key': gateway_keys.RADEMACHER_HOMEPILOT, 'type': 'url'},
    ]

    with db:
        cur = db.cursor()
        cur.execute("SELECT NAME FROM GATEWAYS")

        for gateway in cur.fetchall():
            name = gateway['NAME']
            for i in range(0, len(gateway_types)-1):
                if gateway_types[i]['key'] == name:
                    del gateway_types[i]

        cur.close()

        return gateway_types

def connect_gateway(username, type, ip, db):
    if not has_permission(username, "admin", db):
        return {'result': 'noadmin'}

    if type == gateway_keys.PHILIPS_HUE:
        while True:
            print "try connecting..."

            data = {"devicetype": "homevee#system"}

            response = requests.post("http://" + ip + "/api", data=json.dumps(data))

            result = response.text

            print "result: "+result

            result = json.loads(result)

            result = result[0]

            if "error" in result:
                if result['error']['type'] == 101:
                    return {'result': 'error', 'msg': 'Drücke bitte die Taste auf deinem Philips Hue Gateway.'}
            elif "success" in result:
                user = result['success']['username']

                add_edit_gateway(username, type, user, "", "", ip, "80", "apikey", db)

                return {'result': 'ok'}
    elif type == gateway_keys.MIYO_CUBE:
        while True:
            print "try connecting..."

            response = requests.get("http://" + ip + "/api/link")

            result = response.text

            print "result: "+result

            result = json.loads(result)

            if "error" in result:
                if result['errorLoc'] == "NOTIFY_ERROR_LINKINACTIVE":
                    return {'result': 'error', 'msg': 'Drücke bitte die Taste hinten auf deinem MIYO Cube.'}
            elif "success" in result:
                user = result['success']['username']

                add_edit_gateway(username, type, user, "", "", ip, "80", "apikey", db)

                return {'result': 'ok'}
    else:
        return {'result': 'error'}